@@ -1,5 +1,5 @@
 /*
- * $Id: axis.h,v 1.67.2.2 2013/09/27 00:25:39 sfeam Exp $
+ * $Id: axis.h,v 1.103.2.3 2015/04/04 01:08:32 sfeam Exp $
  *
  */
 
@@ -36,6 +36,7 @@
 #ifndef GNUPLOT_AXIS_H
 #define GNUPLOT_AXIS_H
 
+#include <stddef.h>		/* for offsetof() */
 #include "gp_types.h"		/* for TBOOLEAN */
 
 #include "gadgets.h"
@@ -55,23 +56,29 @@
  * FIRST_X_AXIS & SECOND_AXES == 0
  */
 typedef enum AXIS_INDEX {
-#define FIRST_AXES 0
-    FIRST_Z_AXIS,
+    ALL_AXES = -1,
+    FIRST_Z_AXIS = 0,
+#define FIRST_AXES FIRST_Z_AXIS
     FIRST_Y_AXIS,
     FIRST_X_AXIS,
     COLOR_AXIS,			/* fill gap */
-#define SECOND_AXES 4
     SECOND_Z_AXIS,		/* not used, yet */
+#define SECOND_AXES SECOND_Z_AXIS
     SECOND_Y_AXIS,
     SECOND_X_AXIS,
     POLAR_AXIS,
     T_AXIS,
-    U_AXIS,			/* never used? */
-    V_AXIS			/* ditto */
-#define NO_AXIS 99
+    U_AXIS,
+    V_AXIS,
+    PARALLEL_AXES,	/* The first of up to MAX_PARALLEL_AXES */
+    NO_AXIS = 99
 } AXIS_INDEX;
 
-# define AXIS_ARRAY_SIZE 11
+#ifndef MAX_PARALLEL_AXES
+# define MAX_PARALLEL_AXES MAX_NUM_VAR
+#endif
+# define AXIS_ARRAY_SIZE (11 + MAX_PARALLEL_AXES)
+# define SAMPLE_AXIS SECOND_Z_AXIS
 # define LAST_REAL_AXIS  POLAR_AXIS
 
 /* What kind of ticmarking is wanted? */
@@ -86,7 +93,8 @@ typedef enum en_ticseries_type {
 typedef enum {
     DT_NORMAL=0,		/* default; treat values as pure numeric */
     DT_TIMEDATE,		/* old datatype */
-    DT_DMS			/* degrees minutes seconds */
+    DT_DMS,			/* degrees minutes seconds */
+    DT_UNINITIALIZED
 } td_type;
 
 /* Defines one ticmark for TIC_USER style.
@@ -116,6 +124,7 @@ typedef struct ticdef {
     } def;
     struct position offset;
     TBOOLEAN rangelimited;		/* Limit tics to data range */
+    TBOOLEAN enhanced;			/* Use enhanced text mode or labels */
 } t_ticdef;
 
 /* we want two auto modes for minitics - default where minitics are
@@ -124,16 +133,17 @@ typedef struct ticdef {
  * simply test bit 0 to see if it must do the minitics automatically.
  * similarly, conventional plot can test bit 1 to see if minitics are
  * required */
-enum en_minitics_status {
+typedef enum en_minitics_status {
     MINI_OFF,
     MINI_DEFAULT,
     MINI_USER,
     MINI_AUTO
-};
+} t_minitics_status;
 
 /* Function pointer type for callback functions doing operations for a
  * single ticmark */
-typedef void (*tic_callback) __PROTO((AXIS_INDEX, double, char *, struct lp_style_type,
+typedef void (*tic_callback) __PROTO((AXIS_INDEX, double, char *, int, 
+					struct lp_style_type,
 					struct ticmark *));
 
 /* Values to put in the axis_tics[] variables that decides where the
@@ -147,6 +157,11 @@ typedef void (*tic_callback) __PROTO((AXIS_INDEX, double, char *, struct lp_styl
 #define TICS_MASK      3
 #define TICS_MIRROR    4
 
+/* Tic levels 0 and 1 are maintained in the axis structure.
+ * Tic levels 2 - MAX_TICLEVEL have only one property - scale.
+ */
+#define MAX_TICLEVEL 5
+extern double ticscale[MAX_TICLEVEL];
 
 #if 0 /* HBB 20010806 --- move GRID flags into axis struct */
 /* Need to allow user to choose grid at first and/or second axes tics.
@@ -188,20 +203,16 @@ typedef enum e_constraint {
     CONSTRAINT_UPPER = 1<<1,
     CONSTRAINT_BOTH  = (1<<0 | 1<<1)
 } t_constraint;
-    
 
-/* FIXME 20000725: collect some of those various TBOOLEAN fields into
- * a larger int (or -- shudder -- a bitfield?) */
 typedef struct axis {
 /* range of this axis */
     t_autoscale autoscale;	/* Which end(s) are autoscaled? */
     t_autoscale set_autoscale;	/* what does 'set' think autoscale to be? */
     int range_flags;		/* flag bits about autoscale/writeback: */
     /* write auto-ed ranges back to variables for autoscale */
-#define RANGE_WRITEBACK 1
-    /* allow auto and reversed ranges */
-#define RANGE_REVERSE   2
-    TBOOLEAN range_is_reverted;	/* range [high:low] silently reverted? */
+#define RANGE_WRITEBACK   1
+#define RANGE_SAMPLED     2
+#define RANGE_IS_REVERSED 4
     double min;			/* 'transient' axis extremal values */
     double max;
     double set_min;		/* set/show 'permanent' values */
@@ -228,12 +239,17 @@ typedef struct axis {
     double base;		/* logarithm base value */
     double log_base;		/* ln(base), for easier computations */
 
+/* linked axis information (used only by x2, y2)
+ * If linked_to_primary is TRUE, the primary axis info will be cloned into the
+ * secondary axis only up to this point in the structure.
+ */
+    TBOOLEAN linked_to_primary;
+    struct udft_entry *link_udf;
+
 /* time/date axis control */
-    td_type datatype;		/* DT_NORMAL | DT_TIMEDATE | DT_DMS */
-    TBOOLEAN format_is_numeric;	/* format string looks like numeric??? */
-    char timefmt[MAX_ID_LEN+1];	/* format string for input */
-    char formatstring[MAX_ID_LEN+1];
-				/* the format string for output */
+    td_type datatype;		/* {DT_NORMAL|DT_TIMEDATE} controls _input_ */
+    td_type tictype;		/* {DT_NORMAL|DT_TIMEDATE|DT_DMS} controls _output_ */
+    char *formatstring;		/* the format string for output */
 
 /* ticmark control variables */
     int ticmode;		/* tics on border/axis? mirrored? */
@@ -241,7 +257,7 @@ typedef struct axis {
     int tic_rotate;		/* ticmarks rotated by this angle */
     TBOOLEAN gridmajor;		/* Grid lines wanted on major tics? */
     TBOOLEAN gridminor;		/* Grid lines for minor tics? */
-    int minitics;		/* minor tic mode (none/auto/user)? */
+    t_minitics_status minitics;	/* minor tic mode (none/auto/user)? */
     double mtic_freq;		/* minitic stepsize */
     double ticscale;		/* scale factor for tic marks (was (0..1])*/
     double miniticscale;	/* and for minitics */
@@ -250,15 +266,15 @@ typedef struct axis {
 /* other miscellaneous fields */
     text_label label;		/* label string and position offsets */
     TBOOLEAN manual_justify;	/* override automatic justification */
-    lp_style_type zeroaxis;	/* drawing style for zeroaxis, if any */
+    lp_style_type *zeroaxis;	/* usually points to default_axis_zeroaxis */
 } AXIS;
 
-#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0.0}, {NULL, {0.,0.,0.}, FALSE},  { character, character, character, 0., 0., 0. }, FALSE }
-#define DEFAULT_AXIS_ZEROAXIS {0, LT_NODRAW, 0, 0, 1.0, PTSZ_DEFAULT, FALSE, DEFAULT_COLORSPEC}
+#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0.0}, {NULL, {0.,0.,0.}, FALSE},  { character, character, character, 0., 0., 0. }, FALSE, TRUE }
+#define DEFAULT_AXIS_ZEROAXIS {0, LT_AXIS, 0, DASHTYPE_AXIS, 0, 1.0, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}
 
 #define DEFAULT_AXIS_STRUCT {						    \
 	AUTOSCALE_BOTH, AUTOSCALE_BOTH, /* auto, set_auto */		    \
-	0, FALSE,		/* range_flags, rev_range */		    \
+	0, 			/* range_flags for autoscaling */	    \
 	-10.0, 10.0,		/* 3 pairs of min/max for axis itself */    \
 	-10.0, 10.0,							    \
 	-10.0, 10.0,							    \
@@ -269,8 +285,9 @@ typedef struct axis {
 	0.,        		/* terminal scale */			    \
 	0,        		/* zero axis position */		    \
 	FALSE, 0.0, 0.0,	/* log, base, log(base) */		    \
-	DT_NORMAL, TRUE,	/* datatype, format_numeric */	            \
-	TIMEFMT, DEF_FORMAT,	/* timefmt, output format */		    \
+	FALSE, NULL,		/* linked_to_primary, link function */      \
+	DT_NORMAL,		/* datatype for input */	            \
+	DT_NORMAL, NULL,      	/* tictype for output, output format, */    \
 	NO_TICS,		/* tic output positions (border, mirror) */ \
 	DEFAULT_AXIS_TICDEF,	/* tic series definition */		    \
 	0, FALSE, FALSE, 	/* tic_rotate, grid{major,minor} */	    \
@@ -278,9 +295,12 @@ typedef struct axis {
         1.0, 0.5, TRUE,		/* ticscale, miniticscale, tic_in */	    \
 	EMPTY_LABELSTRUCT,	/* axis label */			    \
 	FALSE,			/* override automatic justification */	    \
-	DEFAULT_AXIS_ZEROAXIS	/* zeroaxis line style */		    \
+	NULL			/* NULL means &default_axis_zeroaxis */	    \
 }
 
+/* This much of the axis structure is cloned by the "set x2range link" command */
+#define AXIS_CLONE_SIZE offsetof(AXIS, linked_to_primary)
+
 /* Table of default behaviours --- a subset of the struct above. Only
  * those fields are present that differ from axis to axis. */
 typedef struct axis_defaults {
@@ -290,8 +310,6 @@ typedef struct axis_defaults {
     int ticmode;		/* tics on border/axis? mirrored? */
 } AXIS_DEFAULTS;
 
-
-
 /* global variables in axis.c */
 
 extern AXIS axis_array[AXIS_ARRAY_SIZE];
@@ -305,10 +323,12 @@ extern const struct gen_table axisname_tbl[AXIS_ARRAY_SIZE+1];
 extern const struct ticdef default_axis_ticdef;
 
 /* default format for tic mark labels */
-#define DEF_FORMAT "% g"
+#define DEF_FORMAT "% h"
+#define DEF_FORMAT_LATEX "$%h$"
 
 /* default parse timedata string */
 #define TIMEFMT "%d/%m/%y,%H:%M"
+extern char *timefmt;
 
 /* axis labels */
 extern const text_label default_axis_label;
@@ -364,9 +384,6 @@ extern AXIS_INDEX x_axis, y_axis, z_axis;
   (((double)(pos)-axis_array[axis].term_lower)/axis_array[axis].term_scale \
    + axis_array[axis].min)
 
-/* these are the old names for these: */
-#define map_x(x) AXIS_MAP(x_axis, x)
-#define map_y(y) AXIS_MAP(y_axis, y)
 
 #define AXIS_SETSCALE(axis, out_low, out_high)			\
     axis_array[axis].term_scale = ((out_high) - (out_low))	\
@@ -442,21 +459,18 @@ do {									\
     this->data_max = -VERYLARGE;					\
 } while(0)
 
-#ifdef VOLATILE_REFRESH
+/* AXIS_INIT2D_REFRESH and AXIS_UPDATE2D_REFRESH(axis) are for volatile data */
 #define AXIS_INIT2D_REFRESH(axis, infinite)				\
 do {									\
     AXIS *this = axis_array + axis;					\
 									\
     this->autoscale = this->set_autoscale;				\
     this->min = (infinite && (this->set_autoscale & AUTOSCALE_MIN))	\
-	? VERYLARGE*1e-3 : AXIS_LOG_VALUE(axis, this->set_min);		\
+	? VERYLARGE : AXIS_LOG_VALUE(axis, this->set_min);		\
     this->max = (infinite && (this->set_autoscale & AUTOSCALE_MAX))	\
-	? -VERYLARGE*1e-3 : AXIS_LOG_VALUE(axis, this->set_max);	\
+	? -VERYLARGE : AXIS_LOG_VALUE(axis, this->set_max);		\
     this->log_base = this->log ? log(this->base) : 0;			\
 } while(0)
-/* why multiply by 1e-3: if an already VERYLARGE x2 and y2 ranges are
-   calculated after zoom-out by mouse, then they would become even larger
-*/
 
 #define AXIS_UPDATE2D_REFRESH(axis)					\
 do {									\
@@ -464,23 +478,9 @@ do {									\
     if ((this_axis->set_autoscale & AUTOSCALE_MIN) == 0)		\
 	this_axis->min = AXIS_LOG_VALUE(axis, this_axis->set_min);	\
     if ((this_axis->set_autoscale & AUTOSCALE_MAX) == 0)		\
-	this_axis->max = AXIS_LOG_VALUE(axis, this_axis->set_max);				\
+	this_axis->max = AXIS_LOG_VALUE(axis, this_axis->set_max);	\
 } while (0)
 
-#endif
-
-/* HBB NEW 20050316: macros to always access the actual minimum, even
- * if 'set view map' or something else flipped things around behind
- * our back */
-#define AXIS_ACTUAL_MIN(axis)				\
-    (axis_array[axis].range_flags & RANGE_REVERSE	\
-     ? axis_array[axis].max : axis_array[axis].min)
-
-#define AXIS_ACTUAL_MAX(axis)				\
-    (axis_array[axis].range_flags & RANGE_REVERSE	\
-     ? axis_array[axis].min : axis_array[axis].max)
-
-
 /* parse a position of the form
  *    [coords] x, [coords] y {,[coords] z}
  * where coords is one of first,second.graph,screen,character
@@ -493,7 +493,7 @@ do {									\
 	struct tm tm;							\
 	double usec;							\
 	char *ss = try_to_get_string();					\
-	if (gstrptime(ss,axis_array[axis].timefmt,&tm,&usec))		\
+	if (gstrptime(ss,timefmt,&tm,&usec))				\
 	    (store) = (double) gtimegm(&tm) + usec;			\
 	free(ss);							\
     } else {								\
@@ -519,83 +519,96 @@ do {							\
  * Note: see the particular implementation for COLOR AXIS below.
  */
 
-#define STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS,	  \
-			        NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)	  \
+#define ACTUAL_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS,  \
+					       NOAUTOSCALE, OUT_ACTION,   \
+					       UNDEF_ACTION, is_cb_axis)  \
 do {									  \
-    if (AXIS == NO_AXIS)                                                  \
-	break;                                                            \
-    /* HBB 20040304: new check to avoid storing infinities and NaNs */	  \
-    if (! (VALUE > -VERYLARGE && VALUE < VERYLARGE)) {			  \
+    struct axis *axis = &axis_array[AXIS];				  \
+    double curval = VALUE;						  \
+    if (AXIS == NO_AXIS)						  \
+	break;								  \
+    /* Version 5: OK to store infinities or NaN */			  \
+    STORE = curval;							  \
+    if (! (curval > -VERYLARGE && curval < VERYLARGE)) {		  \
 	TYPE = UNDEFINED;						  \
 	UNDEF_ACTION;							  \
 	break;								  \
     }									  \
-    if (axis_array[AXIS].log) {						  \
-	if (VALUE<0.0) {						  \
+    if (axis->log) {							  \
+	if (curval < 0.0) {						  \
+	    STORE = not_a_number();					  \
 	    TYPE = UNDEFINED;						  \
 	    UNDEF_ACTION;						  \
 	    break;							  \
-	} else if (VALUE == 0.0) {					  \
+	} else if (curval == 0.0) {					  \
 	    STORE = -VERYLARGE;						  \
 	    TYPE = OUTRANGE;						  \
 	    OUT_ACTION;							  \
 	    break;							  \
 	} else {							  \
-	    STORE = AXIS_DO_LOG(AXIS,VALUE);				  \
+	    STORE = log(curval) / axis->log_base; /* AXIS_DO_LOG() */	  \
 	}								  \
-    } else								  \
-	STORE = VALUE;							  \
+    }									  \
     if (NOAUTOSCALE)							  \
 	break;  /* this plot is not being used for autoscaling */	  \
     if (TYPE != INRANGE)						  \
 	break;  /* don't set y range if x is outrange, for example */	  \
-    if ((int)AXIS < 0)							  \
-	break;	/* HBB 20000507: don't check range if not a coordinate */ \
-    if ( VALUE<axis_array[AXIS].data_min )				  \
-	axis_array[AXIS].data_min = VALUE;				  \
-    if ( VALUE<axis_array[AXIS].min ) {					  \
-	if (axis_array[AXIS].autoscale & AUTOSCALE_MIN)	{		  \
-            if (axis_array[AXIS].min_constraint & CONSTRAINT_LOWER) {     \
-                if (axis_array[AXIS].min_lb <= VALUE) {                   \
-                    axis_array[AXIS].min = VALUE;                         \
-                } else {                                                  \
-                    axis_array[AXIS].min = axis_array[AXIS].min_lb;       \
-                    TYPE = OUTRANGE;                                       \
-                    OUT_ACTION;                                           \
-                    break;                                                \
-                }                                                         \
-            } else {                                                      \
-	        axis_array[AXIS].min = VALUE;				  \
+    if ((! is_cb_axis) && axis->linked_to_primary) {	  		  \
+	axis -= SECOND_AXES;						  \
+	if (axis->link_udf->at) 					  \
+	    curval = eval_link_function(AXIS - SECOND_AXES, curval);	  \
+    } 									  \
+    if ( curval < axis->data_min )					  \
+	axis->data_min = curval;					  \
+    if ( curval < axis->min						  \
+    &&  (curval <= axis->max || axis->max == -VERYLARGE)) {		  \
+	if (axis->autoscale & AUTOSCALE_MIN)	{			  \
+	    if (axis->min_constraint & CONSTRAINT_LOWER) {		  \
+		if (axis->min_lb <= curval) {				  \
+		    axis->min = curval;					  \
+		} else {						  \
+		    axis->min = axis->min_lb;				  \
+		    TYPE = OUTRANGE;					  \
+		    OUT_ACTION;						  \
+		    break;						  \
+		}							  \
+	    } else {							  \
+		axis->min = curval;					  \
 	    }								  \
-	} else {							  \
+	} else if (curval != axis->max) {				  \
 	    TYPE = OUTRANGE;						  \
 	    OUT_ACTION;							  \
 	    break;							  \
 	}								  \
     }									  \
-    if ( VALUE>axis_array[AXIS].data_max )				  \
-	axis_array[AXIS].data_max = VALUE;				  \
-    if ( VALUE>axis_array[AXIS].max ) {					  \
-	if (axis_array[AXIS].autoscale & AUTOSCALE_MAX)	{		  \
-	    if (axis_array[AXIS].max_constraint & CONSTRAINT_UPPER) {     \
-                if (axis_array[AXIS].max_ub >= VALUE) {                   \
-                    axis_array[AXIS].max = VALUE;                         \
-                } else {                                                  \
-                    axis_array[AXIS].max = axis_array[AXIS].max_ub;       \
-                    TYPE =OUTRANGE;                                       \
-                    OUT_ACTION;                                           \
-                    break;                                                \
-                }                                                         \
-            } else {                                                      \
-	        axis_array[AXIS].max = VALUE;                             \
-            }                                                     	  \
-	} else {							  \
+    if ( curval > axis->data_max )					  \
+	axis->data_max = curval;					  \
+    if ( curval > axis->max						  \
+    &&  (curval >= axis->min || axis->min == VERYLARGE)) {		  \
+	if (axis->autoscale & AUTOSCALE_MAX)	{			  \
+	    if (axis->max_constraint & CONSTRAINT_UPPER) {		  \
+		if (axis->max_ub >= curval) {				  \
+		    axis->max = curval;					  \
+		} else {						  \
+		    axis->max = axis->max_ub;				  \
+		    TYPE =OUTRANGE;					  \
+		    OUT_ACTION;						  \
+		    break;						  \
+		}							  \
+	    } else {							  \
+		axis->max = curval;					  \
+	    }								  \
+	} else if (curval != axis->min) {				  \
 	    TYPE = OUTRANGE;						  \
 	    OUT_ACTION;							  \
 	}								  \
     }									  \
 } while(0)
 
+/* normal calls go though this macro, marked as not being a color axis */
+#define STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS, NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)	 \
+ ACTUAL_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS, NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION, 0)
+
 /* Implementation of the above for the color axis. It should not change
  * the type of the point (out-of-range color is plotted with the color
  * of the min or max color value).
@@ -603,9 +616,9 @@ do {									  \
 #define COLOR_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS,	  \
 			       NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)	  \
 {									  \
-    coord_type c_type_tmp = TYPE;						  \
-    STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, c_type_tmp, AXIS,	  \
-			       NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION);	  \
+    coord_type c_type_tmp = TYPE;					  \
+    ACTUAL_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, c_type_tmp, AXIS,	  \
+					   NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION, 1); \
 }
 
 /* Empty macro arguments triggered NeXT cpp bug       */
@@ -613,19 +626,18 @@ do {									  \
 /* Now trying ((void)0) */
 #define NOOP ((void)0)
 
-/* HBB 20000506: new macro to automatically build intializer lists
- * for arrays of AXIS_ARRAY_SIZE equal elements */
+/* HBB 20000506: new macro to automatically build initializer lists
+ * for arrays of AXIS_ARRAY_SIZE=11 equal elements */
 #define AXIS_ARRAY_INITIALIZER(value) {			\
     value, value, value, value, value,			\
 	value, value, value, value, value, value }
 
-/* used by set.c */
-#define SET_DEFFORMAT(axis, flag_array)				\
-	if (flag_array[axis]) {					\
-	    (void) strcpy(axis_array[axis].formatstring,DEF_FORMAT);	\
-	    axis_array[axis].format_is_numeric = 1;		\
-	}
-
+/* FIXME: replace by a subroutine? */
+#define clear_sample_range(axis) do {				\
+	axis_array[SAMPLE_AXIS].range_flags = 0;		\
+	axis_array[SAMPLE_AXIS].min = axis_array[axis].min;	\
+	axis_array[SAMPLE_AXIS].max = axis_array[axis].max;	\
+	} while (0)
 
 /* 'roundoff' check tolerance: less than one hundredth of a tic mark */
 #define SIGNIF (0.01)
@@ -633,10 +645,10 @@ do {									  \
 /* FIXME HBB 20000521: these seem not to be used much, anywhere... */
 #define CheckZero(x,tic) (fabs(x) < ((tic) * SIGNIF) ? 0.0 : (x))
 
-
 /* ------------ functions exported by axis.c */
 t_autoscale load_range __PROTO((AXIS_INDEX, double *, double *, t_autoscale));
 void axis_unlog_interval __PROTO((AXIS_INDEX, double *, double *, TBOOLEAN));
+void axis_revert_range __PROTO((AXIS_INDEX));
 void axis_revert_and_unlog_range __PROTO((AXIS_INDEX));
 double axis_log_value_checked __PROTO((AXIS_INDEX, double, const char *));
 void axis_checked_extend_empty_range __PROTO((AXIS_INDEX, const char *mesg));
@@ -656,23 +668,37 @@ void set_writeback_min __PROTO((AXIS_INDEX));
 void set_writeback_max __PROTO((AXIS_INDEX));
 
 void save_writeback_all_axes __PROTO((void));
-void parse_range __PROTO((AXIS_INDEX axis));
-int  parse_named_range __PROTO((AXIS_INDEX axis, int token));
+int  parse_range __PROTO((AXIS_INDEX axis));
 void parse_skip_range __PROTO((void));
 void check_axis_reversed __PROTO((AXIS_INDEX axis));
 
 /* set widest_tic_label: length of the longest tics label */
-void widest_tic_callback __PROTO((AXIS_INDEX, double place, char *text, 
+void widest_tic_callback __PROTO((AXIS_INDEX, double place, char *text, int ticlevel,
 			struct lp_style_type grid, struct ticmark *));
 
 void get_position __PROTO((struct position *pos));
 void get_position_default __PROTO((struct position *pos, enum position_type default_type));
 
-/* ------------ autoscaling of the color axis */
-#define NEED_PALETTE(plot) \
-   (PM3DSURFACE == (plot)->plot_style \
-    || PM3D_IMPLICIT == pm3d.implicit \
-    || 1 == (plot)->lp_properties.use_palette)
-int set_cbminmax __PROTO((void));
+void gstrdms __PROTO((char *label, char *format, double value));
+
+void clone_linked_axes __PROTO((AXIS_INDEX axis2, AXIS_INDEX axis1));
+
+int map_x __PROTO((double value));
+int map_y __PROTO((double value));
+
+void set_cbminmax __PROTO((void));
+
+#if (defined MAX_PARALLEL_AXES) && (MAX_PARALLEL_AXES > 0)
+char * axis_name __PROTO((AXIS_INDEX));
+#else
+#define axis_name(axis) axis_defaults[axis].name
+#endif
+
+/* macro for tic scale, used in all tic_callback functions */
+#define TIC_SCALE(ticlevel, axis) \
+    (ticlevel <= 0 ? axis_array[axis].ticscale : \
+     ticlevel == 1 ? axis_array[axis].miniticscale : \
+     ticlevel < MAX_TICLEVEL ? ticscale[ticlevel] : \
+     0)
 
 #endif /* GNUPLOT_AXIS_H */
