@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: misc.c,v 1.138.2.19 2014/09/10 04:15:54 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.8 2015/04/25 05:06:02 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - misc.c */
@@ -39,14 +39,18 @@ static char *RCSid() { return RCSid("$Id: misc.c,v 1.138.2.19 2014/09/10 04:15:5
 #include "alloc.h"
 #include "command.h"
 #include "graphics.h"
-#include "parse.h"		/* for const_*() */
 #include "plot.h"
 #include "tables.h"
 #include "util.h"
 #include "variable.h"
 #include "axis.h"
 #include "scanner.h"		/* so that scanner() can count curly braces */
-
+#ifdef _Windows
+# include <fcntl.h>
+# if defined(__WATCOMC__) || defined(__MSC__)
+#  include <io.h>        /* for setmode() */
+# endif
+#endif
 #if defined(HAVE_DIRENT_H)
 # include <sys/types.h>
 # include <dirent.h>
@@ -55,23 +59,17 @@ static char *RCSid() { return RCSid("$Id: misc.c,v 1.138.2.19 2014/09/10 04:15:5
 #endif
 
 static char *recursivefullname __PROTO((const char *path, const char *filename, TBOOLEAN recursive));
-static void prepare_call __PROTO((void));
-static void expand_call_args __PROTO((void));
-
-/* A copy of the declaration from set.c */
-/* There should only be one declaration in a header file. But I do not know
- * where to put it */
-/* void get_position __PROTO((struct position * pos)); */
+static void prepare_call __PROTO((int calltype));
 
 /* State information for load_file(), to recover from errors
  * and properly handle recursive load_file calls
  */
 LFS *lf_head = NULL;		/* NULL if not in load_file */
 
-/* these two could be in load_file, except for error recovery */
-static TBOOLEAN do_load_arg_substitution = FALSE;
-static char *call_args[10] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-static int call_argc;
+/* these are global so that plot.c can load them for the -c option */
+int call_argc;
+char *call_args[10] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+static char *argname[] = {"ARG0","ARG1","ARG2","ARG3","ARG4","ARG5","ARG6","ARG7","ARG8","ARG9"};
 
 /*
  * iso_alloc() allocates a iso_curve structure that can hold 'num'
@@ -138,21 +136,87 @@ iso_free(struct iso_curve *ip)
 }
 
 static void
-prepare_call(void)
+prepare_call(int calltype)
 {
-    call_argc = 0;
-    /* Gnuplot "call" command can have up to 10 arguments "$0" to "$9" */
-    while (!END_OF_COMMAND && call_argc <= 9) {
-	if (isstring(c_token))
-	    m_quote_capture(&call_args[call_argc++], c_token, c_token);
-	else
-	    m_capture(&call_args[call_argc++], c_token, c_token);
-	c_token++;
+    struct udvt_entry *udv;
+    int argindex;
+    if (calltype == 2) {
+	call_argc = 0;
+	while (!END_OF_COMMAND && call_argc <= 9) {
+	    call_args[call_argc] = try_to_get_string();
+	    if (!call_args[call_argc]) {
+		int save_token = c_token;
+
+		/* This catches call "file" STRINGVAR (expression) */
+		if (type_udv(c_token) == STRING) {
+		    call_args[call_argc] = gp_strdup(add_udv(c_token)->udv_value.v.string_val);
+		    c_token++;
+
+		/* Evaluates a parenthesized expression and store the result in a string */
+		} else if (equals(c_token, "(")) {
+		    char val_as_string[32];
+		    struct value a;
+		    const_express(&a);
+		    switch(a.type) {
+			case CMPLX: /* FIXME: More precision? Some way to provide a format? */
+				sprintf(val_as_string, "%g", a.v.cmplx_val.real);
+				call_args[call_argc] = gp_strdup(val_as_string);
+				break;
+			default:
+				int_error(save_token, "Unrecognized argument type");
+				break;
+			case INTGR:	
+				sprintf(val_as_string, "%d", a.v.int_val);
+				call_args[call_argc] = gp_strdup(val_as_string);
+				break;
+		    } 
+
+		/* old (pre version 5) style wrapping of bare tokens as strings */
+		/* is still useful for passing unquoted numbers */
+		} else {
+		    m_capture(&call_args[call_argc], c_token, c_token);
+		    c_token++;
+		}
+	    }
+	    call_argc++;
+	}
+	lf_head->c_token = c_token;
+	if (!END_OF_COMMAND)
+	    int_error(++c_token, "too many arguments for 'call <file>'");
+
+    } else if (calltype == 5) {
+	/* lf_push() moved our call arguments from call_args[] to lf->call_args[] */
+	/* call_argc was determined at program entry */
+	for (argindex = 0; argindex < 10; argindex++) {
+	    call_args[argindex] = lf_head->call_args[argindex];
+	    lf_head->call_args[argindex] = NULL;	/* just to be safe */
+	}
+
+    } else {
+	/* "load" command has no arguments */
+	call_argc = 0;
+    }
+
+    /* Old-style "call" arguments were referenced as $0 ... $9 and $# */
+    /* New-style has ARG0 = script-name, ARG1 ... ARG9 and ARGC */
+    /* FIXME:  If we defined these on entry, we could use get_udv* here */
+    udv = add_udv_by_name("ARGC");
+    Ginteger(&(udv->udv_value), call_argc);
+    udv->udv_undef = FALSE;
+    udv = add_udv_by_name("ARG0");
+    gpfree_string(&(udv->udv_value));
+    Gstring(&(udv->udv_value), gp_strdup(lf_head->name));
+    udv->udv_undef = FALSE;
+    for (argindex = 1; argindex <= 9; argindex++) {
+	char *arg = gp_strdup(call_args[argindex-1]);
+	udv = add_udv_by_name(argname[argindex]);
+	gpfree_string(&(udv->udv_value));
+	Gstring(&(udv->udv_value), arg ? arg : gp_strdup(""));
+	udv->udv_undef = FALSE;
     }
-    if (!END_OF_COMMAND)
-	int_error(++c_token, "too many arguments for 'call <file>'");
 }
 
+#ifdef OLD_STYLE_CALL_ARGS
 
 const char *
 expand_call_arg(int c)
@@ -170,8 +234,11 @@ expand_call_arg(int c)
 	    return "";
 	else
 	    return call_args[ind];
-    } else
-	int_error(NO_CARET, "Invalid substitution $%c", c);
+    } else {
+	/* pass through unrecognized syntax elements that begin with $, e.g. datablock names */
+	sprintf(numstr, "$%c", c);
+	return numstr;
+    }
     return NULL; /* Avoid compiler warning */
 }
 
@@ -206,9 +273,18 @@ expand_call_args(void)
     gp_input_line[il] = '\0';
     free(raw_line);
 }
+#endif /* OLD_STYLE_CALL_ARGS */
 
+/*
+ * load_file() is called from
+ * (1) the "load" command, no arguments substitution is done
+ * (2) the "call" command, arguments are substituted for $0, $1, etc.
+ * (3) on program entry to load initialization files (acts like "load")
+ * (4) to execute script files given on the command line (acts like "load")
+ * (5) to execute a single script file given with -c (acts like "call")
+ */
 void
-load_file(FILE *fp, char *name, TBOOLEAN can_do_args)
+load_file(FILE *fp, char *name, int calltype)
 {
     int len;
 
@@ -216,111 +292,118 @@ load_file(FILE *fp, char *name, TBOOLEAN can_do_args)
     int more;
     int stop = FALSE;
 
+    /* Provide a user-visible copy of the current line number in the input file */
+    udvt_entry *gpval_lineno = add_udv_by_name("GPVAL_LINENO");
+    Ginteger(&gpval_lineno->udv_value, 0);
+    gpval_lineno->udv_undef = FALSE;
+
     lf_push(fp, name, NULL); /* save state for errors and recursion */
-    do_load_arg_substitution = can_do_args;
 
     if (fp == (FILE *) NULL) {
-	os_error(NO_CARET, "Cannot open %s file '%s'",
-		 can_do_args ? "call" : "load", name);
-    } else if (fp == stdin) {
+	int_error(NO_CARET, "Cannot open script file '%s'", name);
+	return; /* won't actually reach here */
+    }
+
+    if (fp == stdin) {
 	/* DBT 10-6-98  go interactive if "-" named as load file */
 	interactive = TRUE;
 	while (!com_line());
-    } else {
-	call_argc = 0;
-	if (can_do_args) {
-	    prepare_call();
-	    lf_head->c_token = c_token; /* update after prepare_call() */
-	}
-	/* things to do after lf_push */
-	inline_num = 0;
-	/* go into non-interactive mode during load */
-	/* will be undone below, or in load_file_error */
-	interactive = FALSE;
+	(void) lf_pop();
+	return;
+    }
 
-	while (!stop) {	/* read all lines in file */
-	    left = gp_input_line_len;
-	    start = 0;
-	    more = TRUE;
+    /* We actually will read from a file */
+    prepare_call(calltype);
 
-	    /* read one logical line */
-	    while (more) {
-		if (fgets(&(gp_input_line[start]), left, fp) == (char *) NULL) {
-		    stop = TRUE;	/* EOF in file */
-		    gp_input_line[start] = '\0';
-		    more = FALSE;
-		} else {
-		    inline_num++;
-		    len = strlen(gp_input_line) - 1;
-		    if (gp_input_line[len] == '\n') {	/* remove any newline */
-			gp_input_line[len] = '\0';
-			/* Look, len was 1-1 = 0 before, take care here! */
+    /* things to do after lf_push */
+    inline_num = 0;
+    /* go into non-interactive mode during load */
+    /* will be undone below, or in load_file_error */
+    interactive = FALSE;
+
+    while (!stop) {	/* read all lines in file */
+	left = gp_input_line_len;
+	start = 0;
+	more = TRUE;
+
+	/* read one logical line */
+	while (more) {
+	    if (fgets(&(gp_input_line[start]), left, fp) == (char *) NULL) {
+		stop = TRUE;	/* EOF in file */
+		gp_input_line[start] = '\0';
+		more = FALSE;
+	    } else {
+		inline_num++;
+		gpval_lineno->udv_value.v.int_val = inline_num;	/* User visible copy */
+		len = strlen(gp_input_line) - 1;
+		if (gp_input_line[len] == '\n') {	/* remove any newline */
+		    gp_input_line[len] = '\0';
+		    /* Look, len was 1-1 = 0 before, take care here! */
+		    if (len > 0)
+			--len;
+		    if (gp_input_line[len] == '\r') {	/* remove any carriage return */
+			gp_input_line[len] = NUL;
 			if (len > 0)
 			    --len;
-			if (gp_input_line[len] == '\r') {	/* remove any carriage return */
-			    gp_input_line[len] = NUL;
-			    if (len > 0)
-				--len;
-			}
-		    } else if (len + 2 >= left) {
-			extend_input_line();
-			left = gp_input_line_len - len - 1;
-			start = len + 1;
-			continue;	/* don't check for '\' */
 		    }
-		    if (gp_input_line[len] == '\\') {
-			/* line continuation */
-			start = len;
-			left = gp_input_line_len - start;
-		    } else {
-			/* EAM May 2011 - handle multi-line bracketed clauses {...}.
-			 * Introduces a requirement for scanner.c and scanner.h
-			 * This code is redundant with part of do_line(),
-			 * but do_line() assumes continuation lines come from stdin.
-			 */
-
-			/* macros in a clause are problematic, as they are */
-			/* only expanded once even if the clause is replayed */
-			string_expand_macros();
-
-			/* Strip off trailing comment and count curly braces */
-			num_tokens = scanner(&gp_input_line, &gp_input_line_len);
-			if (gp_input_line[token[num_tokens].start_index] == '#') {
-			    gp_input_line[token[num_tokens].start_index] = NUL;
-			    start = token[num_tokens].start_index;
-			    left = gp_input_line_len - start;
-			}
-			/* Read additional lines if necessary to complete a
-			 * bracketed clause {...}
-			 */
-			if (curly_brace_count < 0)
-			    int_error(NO_CARET, "Unexpected }");
-			if (curly_brace_count > 0) {
-			    if (len + 4 > gp_input_line_len)
-				extend_input_line();
-			    strcat(gp_input_line,";\n");
-			    start = strlen(gp_input_line);
-			    left = gp_input_line_len - start;
-			    continue;
-			}
-			
-			more = FALSE;
-		    }
-
+		} else if (len + 2 >= left) {
+		    extend_input_line();
+		    left = gp_input_line_len - len - 1;
+		    start = len + 1;
+		    continue;	/* don't check for '\' */
 		}
-	    }
+		if (gp_input_line[len] == '\\') {
+		    /* line continuation */
+		    start = len;
+		    left = gp_input_line_len - start;
+		} else {
+		    /* EAM May 2011 - handle multi-line bracketed clauses {...}.
+		     * Introduces a requirement for scanner.c and scanner.h
+		     * This code is redundant with part of do_line(),
+		     * but do_line() assumes continuation lines come from stdin.
+		     */
 
-	    /* process line */
-	    if (strlen(gp_input_line) > 0) {
-		if (can_do_args)
-		  expand_call_args();
+		    /* macros in a clause are problematic, as they are */
+		    /* only expanded once even if the clause is replayed */
+		    string_expand_macros();
+
+		    /* Strip off trailing comment and count curly braces */
+		    num_tokens = scanner(&gp_input_line, &gp_input_line_len);
+		    if (gp_input_line[token[num_tokens].start_index] == '#') {
+			gp_input_line[token[num_tokens].start_index] = NUL;
+			start = token[num_tokens].start_index;
+			left = gp_input_line_len - start;
+		    }
+		    /* Read additional lines if necessary to complete a
+		     * bracketed clause {...}
+		     */
+		    if (curly_brace_count < 0)
+			int_error(NO_CARET, "Unexpected }");
+		    if (curly_brace_count > 0) {
+			if ((len + 4) > gp_input_line_len)
+			    extend_input_line();
+			strcat(gp_input_line,";\n");
+			start = strlen(gp_input_line);
+			left = gp_input_line_len - start;
+			continue;
+		    }
+		    
+		    more = FALSE;
+		}
 
-		screen_ok = FALSE;	/* make sure command line is
-					   echoed on error */
-		if (do_line())
-		    stop = TRUE;
 	    }
 	}
+
+	/* process line */
+	if (strlen(gp_input_line) > 0) {
+#ifdef OLD_STYLE_CALL_ARGS
+	    if (calltype == 2 || calltype == 5)
+		expand_call_args();
+#endif
+	    screen_ok = FALSE;	/* make sure command line is echoed on error */
+	    if (do_line())
+		stop = TRUE;
+	}
     }
 
     /* pop state */
@@ -335,6 +418,7 @@ lf_pop()
 {
     LFS *lf;
     int argindex;
+    struct udvt_entry *udv;
 
     if (lf_head == NULL)
 	return (FALSE);
@@ -350,15 +434,37 @@ lf_pop()
     else
 	fclose(lf->fp);
 
-    for (argindex = 0; argindex < 10; argindex++) {
-	if (call_args[argindex])
-	    free(call_args[argindex]);
-	call_args[argindex] = lf->call_args[argindex];
+    /* call arguments are not relevant when invoked from do_string_and_free */
+    if (lf->cmdline == NULL) {
+	for (argindex = 0; argindex < 10; argindex++) {
+	    if (call_args[argindex])
+		free(call_args[argindex]);
+	    call_args[argindex] = lf->call_args[argindex];
+	}
+	call_argc = lf->call_argc;
+
+	/* Restore ARGC and ARG0 ... ARG9 */
+	if ((udv = get_udv_by_name("ARGC"))) {
+	    Ginteger(&(udv->udv_value), call_argc);
+	}
+	if ((udv = get_udv_by_name("ARG0"))) {
+	    gpfree_string(&(udv->udv_value));
+	    Gstring(&(udv->udv_value),
+		(lf->prev && lf->prev->name) ? gp_strdup(lf->prev->name) : gp_strdup(""));
+	}
+	for (argindex = 1; argindex <= 9; argindex++) {
+	    if ((udv = get_udv_by_name(argname[argindex]))) {
+		gpfree_string(&(udv->udv_value));
+		Gstring(&(udv->udv_value), gp_strdup(call_args[argindex-1]));
+		if (!call_args[argindex-1])
+		    udv->udv_undef = TRUE;
+	    }
+	}
     }
-    call_argc = lf->call_argc;
-    do_load_arg_substitution = lf->do_load_arg_substitution;
+
     interactive = lf->interactive;
     inline_num = lf->inline_num;
+    add_udv_by_name("GPVAL_LINENO")->udv_value.v.int_val = inline_num;
     if_depth = lf->if_depth;
     if_condition = lf->if_condition;
     if_open_for_else = lf->if_open_for_else;
@@ -384,9 +490,13 @@ lf_pop()
     return (TRUE);
 }
 
-/* push onto load_file state stack
-   essentially, we save information needed to undo the load_file changes
-   called by load_file */
+/* lf_push is called from two different contexts:
+ *    load_file passes fp and file name (3rd param NULL)
+ *    do_string_and_free passes cmdline (1st and 2nd params NULL)
+ * In either case the routines lf_push/lf_pop save and restore state
+ * information that may be changed by executing commands from a file
+ * or from the passed command line.
+ */
 void
 lf_push(FILE *fp, char *name, char *cmdline)
 {
@@ -405,11 +515,14 @@ lf_push(FILE *fp, char *name, char *cmdline)
 
     lf->interactive = interactive;	/* save current state */
     lf->inline_num = inline_num;	/* save current line number */
-    lf->do_load_arg_substitution = do_load_arg_substitution;
     lf->call_argc = call_argc;
-    for (argindex = 0; argindex < 10; argindex++) {
-	lf->call_args[argindex] = call_args[argindex];
-	call_args[argindex] = NULL;	/* initially no args */
+
+    /* Call arguments are irrelevant if invoked from do_string_and_free */
+    if (cmdline == NULL) {
+	for (argindex = 0; argindex < 10; argindex++) {
+	    lf->call_args[argindex] = call_args[argindex];
+	    call_args[argindex] = NULL;	/* initially no args */
+	}
     }
     lf->depth = lf_head ? lf_head->depth+1 : 0;	/* recursion depth */
     if (lf->depth > STACK_DEPTH)
@@ -446,12 +559,6 @@ load_file_error()
     while (lf_pop());
 }
 
-/* find max len of keys and count keys with len > 0 */
-
-/* FIXME HBB 2000508: by design, this one belongs into 'graphics', and the
- * next to into 'graph3d'. Actually, the existence of a module like this
- * 'misc' is almost always a sign of bad design, IMHO */
-/* may return NULL */
 FILE *
 loadpath_fopen(const char *filename, const char *mode)
 {
@@ -489,6 +596,10 @@ loadpath_fopen(const char *filename, const char *mode)
 
     }
 
+#ifdef _Windows
+    if (fp != NULL)
+	setmode(fileno(fp), _O_BINARY);
+#endif
     return fp;
 }
 
@@ -689,7 +800,7 @@ get_style()
 
     c_token++;
 
-    if (ps == -1)
+    if (ps == PLOT_STYLE_NONE)
 	int_error(c_token, "unrecognized plot type");
 
     return ps;
@@ -788,16 +899,107 @@ need_fill_border(struct fill_style_type *fillstyle)
     return TRUE;
 }
 
+int
+parse_dashtype(struct t_dashtype *dt)
+{
+    int res = DASHTYPE_SOLID;
+    int j = 0;
+    int k = 0;
+    char *dash_str = NULL;
+
+    /* Erase any previous contents */
+    memset(dt, 0, sizeof(struct t_dashtype));
+
+    /* Fill in structure based on keyword ... */ 
+    if (equals(c_token, "solid")) {
+	res = DASHTYPE_SOLID;
+	c_token++;
+
+    /* Or numerical pattern consisting of pairs solid,empty,solid,empty... */
+    } else if (equals(c_token, "(")) {
+	c_token++;
+	while (!END_OF_COMMAND) {
+	    if (j >= DASHPATTERN_LENGTH) {
+		int_error(c_token, "too many pattern elements");
+	    }
+	    dt->pattern[j++] = real_expression();	/* The solid portion */
+	    if (!equals(c_token++, ","))
+		int_error(c_token, "expecting comma");
+	    dt->pattern[j++] = real_expression();	/* The empty portion */
+	    if (equals(c_token, ")"))
+		break;
+	    if (!equals(c_token++, ","))
+		int_error(c_token, "expecting comma");
+	}
+
+	if (!equals(c_token, ")"))
+	    int_error(c_token, "expecting , or )");
+
+	c_token++;
+	res = DASHTYPE_CUSTOM;
+
+    /* Or string representing pattern elements ... */
+    } else if ((dash_str = try_to_get_string())) {
+#define DSCALE 10.
+	while (dash_str[j] && (k < DASHPATTERN_LENGTH || dash_str[j] == ' ')) {
+	    /* .      Dot with short space 
+	     * -      Dash with regular space
+	     * _      Long dash with regular space
+	     * space  Don't add new dash, just increase last space */
+	    switch (dash_str[j]) {
+	    case '.':
+		dt->pattern[k++] = 0.2 * DSCALE;
+		dt->pattern[k++] = 0.5 * DSCALE;
+		break;
+	    case '-':
+		dt->pattern[k++] = 1.0 * DSCALE;
+		dt->pattern[k++] = 1.0 * DSCALE;
+		break;
+	    case '_':
+		dt->pattern[k++] = 2.0 * DSCALE;
+		dt->pattern[k++] = 1.0 * DSCALE;
+		break;
+	    case ' ':
+		if (k > 0)
+		dt->pattern[k-1] += 1.0 * DSCALE;
+		break;
+	    default:
+		int_error(c_token - 1, "expecting one of . - _ or space");
+	    }
+	    j++;
+#undef  DSCALE
+	}
+	/* truncate dash_str if we ran out of space in the array representation */
+	dash_str[j] = '\0';
+	strncpy(dt->dstring, dash_str, sizeof(dt->dstring)-1);
+	free(dash_str);
+	res = DASHTYPE_CUSTOM;
+
+    /* Or index of previously defined dashtype */
+    /* FIXME: Is the index enough or should we copy its contents into this one? */
+    /* FIXME: What happens if there is a recursive definition? */
+    } else {
+	res = int_expression() - 1;
+	if (res < 0)
+	    int_error(c_token - 1, "tag must be > 0");
+    }
+
+    return res;
+}
+
 /*
- * allow_ls controls whether we are allowed to accept linestyle in
- * the current context [ie not when doing a  set linestyle command]
- * allow_point is whether we accept a point command
+ * destination_class tells us whether we are filling in a line style ('set style line'),
+ * a persistant linetype ('set linetype') or an ad hoc set of properties for a single
+ * use ('plot ... lc foo lw baz').
+ * allow_point controls whether we accept a point attribute in this lp_style.
  */
 int
-lp_parse(struct lp_style_type *lp, TBOOLEAN allow_ls, TBOOLEAN allow_point)
+lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_point)
 {
     /* keep track of which options were set during this call */
-    int set_lt = 0, set_pal = 0, set_lw = 0, set_pt = 0, set_ps = 0, set_pi = 0;
+    int set_lt = 0, set_pal = 0, set_lw = 0; 
+    int set_pt = 0, set_ps  = 0, set_pi = 0;
+    int set_dt = 0;
     int new_lt = 0;
 
     /* EAM Mar 2010 - We don't want properties from a user-defined default
@@ -807,34 +1009,47 @@ lp_parse(struct lp_style_type *lp, TBOOLEAN allow_ls, TBOOLEAN allow_point)
      */
     struct lp_style_type newlp = *lp;
 	
-	if (allow_ls &&
-	    (almost_equals(c_token, "lines$tyle") || equals(c_token, "ls"))) {
-	    c_token++;
-	    lp_use_properties(lp, int_expression());
-	} 
+    if ((destination_class == LP_ADHOC)
+    && (almost_equals(c_token, "lines$tyle") || equals(c_token, "ls"))) {
+	c_token++;
+	lp_use_properties(lp, int_expression());
+    } 
     
-	while (!END_OF_COMMAND) {
-	    if (almost_equals(c_token, "linet$ype") || equals(c_token, "lt")) {
-		if (set_lt++)
+    while (!END_OF_COMMAND) {
+
+	/* This special case is to flag an attemp to "set object N lt <lt>",
+	 * which would otherwise be accepted but ignored, leading to confusion
+	 * FIXME:  Couldn't this be handled at a higher level?
+	 */
+	if ((destination_class == LP_NOFILL)
+	&&  (equals(c_token,"lt") || almost_equals(c_token,"linet$ype"))) {
+	    int_error(c_token, "object linecolor must be set using fillstyle border");
+	}
+
+	if (almost_equals(c_token, "linet$ype") || equals(c_token, "lt")) {
+	    if (set_lt++)
+		break;
+	    if (destination_class == LP_TYPE)
+		int_error(c_token, "linetype definition cannot use linetype");
+	    c_token++;
+	    if (almost_equals(c_token, "rgb$color")) {
+		if (set_pal++)
 		    break;
-		c_token++;
-		if (almost_equals(c_token, "rgb$color")) {
-		    if (set_pal++)
-			break;
-		    c_token--;
-		    parse_colorspec(&(newlp.pm3d_color), TC_RGB);
-		    newlp.use_palette = 1;
-		} else
+		c_token--;
+		parse_colorspec(&(newlp.pm3d_color), TC_RGB);
+	    } else
 		/* both syntaxes allowed: 'with lt pal' as well as 'with pal' */
 		if (almost_equals(c_token, "pal$ette")) {
 		    if (set_pal++)
 			break;
 		    c_token--;
 		    parse_colorspec(&(newlp.pm3d_color), TC_Z);
-		    newlp.use_palette = 1;
 		} else if (equals(c_token,"bgnd")) {
 		    *lp = background_lp;
 		    c_token++;
+		} else if (equals(c_token,"black")) {
+		    *lp = default_border_lp;
+		    c_token++;
 		} else if (equals(c_token,"nodraw")) {
 		    lp->l_type = LT_NODRAW;
 		    c_token++;
@@ -843,159 +1058,221 @@ lp_parse(struct lp_style_type *lp, TBOOLEAN allow_ls, TBOOLEAN allow_point)
 		    new_lt = int_expression();
 		    lp->l_type = new_lt - 1;
 		    /* user may prefer explicit line styles */
-		    if (prefer_line_styles && allow_ls)
+		    if (prefer_line_styles && (destination_class != LP_STYLE))
 			lp_use_properties(lp, new_lt);
 		    else
 			load_linetype(lp, new_lt);
 		}
-	    } /* linetype, lt */
+	} /* linetype, lt */
 
-	    /* both syntaxes allowed: 'with lt pal' as well as 'with pal' */
-	    if (almost_equals(c_token, "pal$ette")) {
-		if (set_pal++)
-		    break;
+	/* both syntaxes allowed: 'with lt pal' as well as 'with pal' */
+	if (almost_equals(c_token, "pal$ette")) {
+	    if (set_pal++)
+		break;
+	    c_token--;
+	    parse_colorspec(&(newlp.pm3d_color), TC_Z);
+	    continue;
+	}
+
+	/* This is so that "set obj ... lw N fc <colorspec>" doesn't eat */
+	/* up the colorspec as a line property.  We need to parse it later */
+	/* as a _fill_ property */
+	if ((destination_class == LP_NOFILL)
+	&&  (equals(c_token,"fc") || almost_equals(c_token,"fillc$olor")))
+	    break;
+
+	if (equals(c_token,"lc") || almost_equals(c_token,"linec$olor")
+	||  equals(c_token,"fc") || almost_equals(c_token,"fillc$olor")
+	   ) {
+	    if (set_pal++)
+		break;
+	    c_token++;
+	    if (almost_equals(c_token, "rgb$color") || isstring(c_token)) {
+		c_token--;
+		parse_colorspec(&(newlp.pm3d_color), TC_RGB);
+	    } else if (almost_equals(c_token, "pal$ette")) {
 		c_token--;
 		parse_colorspec(&(newlp.pm3d_color), TC_Z);
-		newlp.use_palette = 1;
-		continue;
-	    }
-
-	    if (equals(c_token,"lc") || almost_equals(c_token,"linec$olor")
-	    ||  equals(c_token,"fc") || almost_equals(c_token,"fillc$olor")) {
-		newlp.use_palette = 1;
-		if (set_pal++)
-		    break;
+	    } else if (equals(c_token,"bgnd")) {
+		newlp.pm3d_color.type = TC_LT;
+		newlp.pm3d_color.lt = LT_BACKGROUND;
 		c_token++;
-		if (almost_equals(c_token, "rgb$color") || isstring(c_token)) {
-		    c_token--;
-		    parse_colorspec(&(newlp.pm3d_color), TC_RGB);
-		} else if (almost_equals(c_token, "pal$ette")) {
-		    c_token--;
-		    parse_colorspec(&(newlp.pm3d_color), TC_Z);
-		} else if (equals(c_token,"bgnd")) {
+	    } else if (equals(c_token,"black")) {
+		newlp.pm3d_color.type = TC_LT;
+		newlp.pm3d_color.lt = LT_BLACK;
+		c_token++;
+	    } else if (almost_equals(c_token, "var$iable")) {
+		c_token++;
+		newlp.l_type = LT_COLORFROMCOLUMN;
+		newlp.pm3d_color.type = TC_LINESTYLE;
+	    } else {
+		/* Pull the line colour from a default linetype, but */
+		/* only if we are not in the middle of defining one! */
+		if (destination_class != LP_STYLE) {
+		    struct lp_style_type temp;
+		    load_linetype(&temp, int_expression());
+		    newlp.pm3d_color = temp.pm3d_color;
+		} else {
 		    newlp.pm3d_color.type = TC_LT;
-		    newlp.pm3d_color.lt = LT_BACKGROUND;
-		    c_token++;
-		} else if (almost_equals(c_token, "var$iable")) {
-		    c_token++;
-		    newlp.l_type = LT_COLORFROMCOLUMN;
-		    newlp.pm3d_color.type = TC_LINESTYLE;
-		} else {
-		    /* Pull the line colour from a default linetype, but */
-		    /* only if we are not in the middle of defining one! */
-		    if (allow_ls) {
-			struct lp_style_type temp;
-			load_linetype(&temp, int_expression());
-			newlp.pm3d_color = temp.pm3d_color;
-		    } else {
-			newlp.pm3d_color.type = TC_LT;
-			newlp.pm3d_color.lt = int_expression() - 1;
-		    }
+		    newlp.pm3d_color.lt = int_expression() - 1;
 		}
-		continue;
 	    }
+	    continue;
+	}
 
-	    if (almost_equals(c_token, "linew$idth") || equals(c_token, "lw")) {
-		if (set_lw++)
+	if (almost_equals(c_token, "linew$idth") || equals(c_token, "lw")) {
+	    if (set_lw++)
+		break;
+	    c_token++;
+	    newlp.l_width = real_expression();
+	    if (newlp.l_width < 0)
+		newlp.l_width = 0;
+	    continue;
+	}
+
+	if (equals(c_token,"bgnd")) {
+	    if (set_lt++)
+		break;;
+	    c_token++;
+	    *lp = background_lp;
+	    continue;
+	}
+
+	if (equals(c_token,"black")) {
+	    if (set_lt++)
+		break;;
+	    c_token++;
+	    *lp = default_border_lp;
+	    continue;
+	}
+
+	if (almost_equals(c_token, "pointt$ype") || equals(c_token, "pt")) {
+	    if (allow_point) {
+		char *symbol;
+		if (set_pt++)
 		    break;
 		c_token++;
-		newlp.l_width = real_expression();
-		if (newlp.l_width < 0)
-		    newlp.l_width = 0;
-		continue;
-	    }
-
-	    if (equals(c_token,"bgnd")) {
-		if (set_lt++)
-		    break;;
-		c_token++;
-		*lp = background_lp;
-		continue;
-	    }
-
-	    if (almost_equals(c_token, "pointt$ype") || equals(c_token, "pt")) {
-		if (allow_point) {
-		    if (set_pt++)
-			break;
-		    c_token++;
+		if ((symbol = try_to_get_string())) {
+		    newlp.p_type = PT_CHARACTER;
+		    /* An alternative mechanism would be to use
+		     * utf8toulong(&newlp.p_char, symbol);
+		     */
+		    strncpy((char *)(&newlp.p_char), symbol, 3);
+		    /* Truncate ascii text to single character */
+		    if ((((char *)&newlp.p_char)[0] & 0x80) == 0)
+			((char *)&newlp.p_char)[1] = '\0';
+		    /* UTF-8 characters may use up to 3 bytes */
+		    ((char *)&newlp.p_char)[3] = '\0';
+		    free(symbol);
+		} else {
 		    newlp.p_type = int_expression() - 1;
-		} else {
-		    int_warn(c_token, "No pointtype specifier allowed, here");
-		    c_token += 2;
 		}
-		continue;
+	    } else {
+		int_warn(c_token, "No pointtype specifier allowed, here");
+		c_token += 2;
 	    }
+	    continue;
+	}
 
-	    if (almost_equals(c_token, "points$ize") || equals(c_token, "ps")) {
-		if (allow_point) {
-		    if (set_ps++)
-			break;
-		    c_token++;
-		    if (almost_equals(c_token, "var$iable")) {
-			newlp.p_size = PTSZ_VARIABLE;
-			c_token++;
-		    } else if (almost_equals(c_token, "def$ault")) {
-			newlp.p_size = PTSZ_DEFAULT;
-			c_token++;
-		    } else {
-			newlp.p_size = real_expression();
-			if (newlp.p_size < 0)
-			    newlp.p_size = 0;
-		    }
-		} else {
-		    int_warn(c_token, "No pointsize specifier allowed, here");
-		    c_token += 2;
-		}
-		continue;
-	    }
-
-	    if (almost_equals(c_token, "pointi$nterval") || equals(c_token, "pi")) {
+	if (almost_equals(c_token, "points$ize") || equals(c_token, "ps")) {
+	    if (allow_point) {
+		if (set_ps++)
+		    break;
 		c_token++;
-		if (allow_point) {
-		    newlp.p_interval = int_expression();
-		    set_pi = 1;
+		if (almost_equals(c_token, "var$iable")) {
+		    newlp.p_size = PTSZ_VARIABLE;
+		    c_token++;
+		} else if (almost_equals(c_token, "def$ault")) {
+		    newlp.p_size = PTSZ_DEFAULT;
+		    c_token++;
 		} else {
-		    int_warn(c_token, "No pointinterval specifier allowed, here");
-		    int_expression();
+		    newlp.p_size = real_expression();
+		    if (newlp.p_size < 0)
+			newlp.p_size = 0;
 		}
-		continue;
+	    } else {
+		int_warn(c_token, "No pointsize specifier allowed, here");
+		c_token += 2;
 	    }
-
-
-	    /* caught unknown option -> quit the while(1) loop */
-	    break;
+	    continue;
 	}
 
-	if (set_lt > 1 || set_pal > 1 || set_lw > 1 || set_pt > 1 || set_ps > 1)
-	    int_error(c_token, "duplicated arguments in style specification");
-
-	if (set_pal) {
-	    lp->pm3d_color = newlp.pm3d_color;
-	    lp->use_palette = newlp.use_palette;
-	    /* FIXME: This was used by hidden3d, but breaks contour coloring */
-	    /* new_lt = LT_SINGLECOLOR; */
+	if (almost_equals(c_token, "pointi$nterval") || equals(c_token, "pi")) {
+	    c_token++;
+	    if (allow_point) {
+		newlp.p_interval = int_expression();
+		set_pi = 1;
+	    } else {
+		int_warn(c_token, "No pointinterval specifier allowed, here");
+		int_expression();
+	    }
+	    continue;
 	}
-	if (set_lw)
-	    lp->l_width = newlp.l_width;
-	if (set_pt)
-	    lp->p_type = newlp.p_type;
-	if (set_ps)
-	    lp->p_size = newlp.p_size;
-	if (set_pi)
-	    lp->p_interval = newlp.p_interval;
-	if (newlp.l_type == LT_COLORFROMCOLUMN)
-	    lp->l_type = LT_COLORFROMCOLUMN;
+
+	if (almost_equals(c_token, "dasht$ype") || equals(c_token, "dt")) {
+	    int tmp;
+	    if (set_dt++)
+		break;
+	    c_token++;
+	    tmp = parse_dashtype(&newlp.custom_dash_pattern);
+	    /* Pull the dashtype from the list of already defined dashtypes, */
+	    /* but only if it we didn't get an explicit one back from parse_dashtype */ 
+	    if (tmp >= 0)
+		tmp = load_dashtype(&newlp.custom_dash_pattern, tmp + 1);
+	    newlp.d_type = tmp;
+	    continue;
+	}
+
+
+	/* caught unknown option -> quit the while(1) loop */
+	break;
+    }
+
+    if (set_lt > 1 || set_pal > 1 || set_lw > 1 || set_pt > 1 || set_ps > 1 || set_dt > 1)
+	int_error(c_token, "duplicated arguments in style specification");
+
+    if (set_pal) {
+	lp->pm3d_color = newlp.pm3d_color;
+	/* hidden3d uses this to decide that a single color surface is wanted */
+	lp->flags |= LP_EXPLICIT_COLOR;
+    } else {
+	lp->flags &= ~LP_EXPLICIT_COLOR;
+    }
+    if (set_lw)
+	lp->l_width = newlp.l_width;
+    if (set_pt) {
+	lp->p_type = newlp.p_type;
+	lp->p_char = newlp.p_char;
+    }
+    if (set_ps)
+	lp->p_size = newlp.p_size;
+    if (set_pi)
+	lp->p_interval = newlp.p_interval;
+    if (newlp.l_type == LT_COLORFROMCOLUMN)
+	lp->l_type = LT_COLORFROMCOLUMN;
+    if (set_dt) {
+	lp->d_type = newlp.d_type;
+	lp->custom_dash_pattern = newlp.custom_dash_pattern;
+    }	
+		
 
     return new_lt;
 }
 
 /* <fillstyle> = {empty | solid {<density>} | pattern {<n>}} {noborder | border {<lt>}} */
+const struct gen_table fs_opt_tbl[] = {
+    {"e$mpty", FS_EMPTY},
+    {"s$olid", FS_SOLID},
+    {"p$attern", FS_PATTERN},
+    {NULL, -1}
+};
+
 void
 parse_fillstyle(struct fill_style_type *fs, int def_style, int def_density, int def_pattern, 
 		t_colorspec def_bordertype)
 {
     TBOOLEAN set_fill = FALSE;
-    TBOOLEAN set_param = FALSE;
+    TBOOLEAN set_border = FALSE;
     TBOOLEAN transparent = FALSE;
 
     /* Set defaults */
@@ -1011,29 +1288,55 @@ parse_fillstyle(struct fill_style_type *fs, int def_style, int def_density, int
     c_token++;
 
     while (!END_OF_COMMAND) {
+	int i;
+
 	if (almost_equals(c_token, "trans$parent")) {
 	    transparent = TRUE;
 	    c_token++;
-	}
-
-	if (almost_equals(c_token, "e$mpty")) {
-	    fs->fillstyle = FS_EMPTY;
-	    c_token++;
-	} else if (almost_equals(c_token, "s$olid")) {
-	    fs->fillstyle = transparent ? FS_TRANSPARENT_SOLID : FS_SOLID;
-	    set_fill = TRUE;
-	    c_token++;
-	} else if (almost_equals(c_token, "p$attern")) {
-	    fs->fillstyle = transparent ? FS_TRANSPARENT_PATTERN : FS_PATTERN;
-	    set_fill = TRUE;
-	    c_token++;
-	}
-
-	if (END_OF_COMMAND)
 	    continue;
-	else if (almost_equals(c_token, "bo$rder")) {
+	}
+
+	i = lookup_table(fs_opt_tbl, c_token);
+	switch (i) {
+	    default:
+		 break;
+
+	    case FS_EMPTY:
+	    case FS_SOLID:
+	    case FS_PATTERN:
+
+		if (set_fill && fs->fillstyle != i)
+		    int_error(c_token, "conflicting option");
+		fs->fillstyle = i;
+		set_fill = TRUE;
+		c_token++;
+		
+		if (isanumber(c_token) || type_udv(c_token) == INTGR || type_udv(c_token) == CMPLX) {
+		    if (fs->fillstyle == FS_SOLID) {
+			/* user sets 0...1, but is stored as an integer 0..100 */
+			fs->filldensity = 100.0 * real_expression() + 0.5;
+			if (fs->filldensity < 0)
+			    fs->filldensity = 0;
+			if (fs->filldensity > 100)
+			    fs->filldensity = 100;
+		    } else if (fs->fillstyle == FS_PATTERN) {
+			fs->fillpattern = int_expression();
+			if (fs->fillpattern < 0)
+			    fs->fillpattern = 0;
+		    } else
+			int_error(c_token, "this fill style does not have a parameter");
+		}
+		continue;
+	}
+
+	if (almost_equals(c_token, "bo$rder")) {
+	    if (set_border && fs->border_color.lt == LT_NODRAW)
+		int_error(c_token, "conflicting option");
 	    fs->border_color.type = TC_DEFAULT;
+	    set_border = TRUE;
 	    c_token++;
+	    if (END_OF_COMMAND)
+		continue;
 	    if (equals(c_token,"-") || isanumber(c_token)) {
 		fs->border_color.type = TC_LT;
 		fs->border_color.lt = int_expression() - 1;
@@ -1046,32 +1349,23 @@ parse_fillstyle(struct fill_style_type *fs, int def_style, int def_density, int
 	    }
 	    continue;
 	} else if (almost_equals(c_token, "nobo$rder")) {
+	    if (set_border && fs->border_color.lt != LT_NODRAW)
+		int_error(c_token, "conflicting option");
 	    fs->border_color.type = TC_LT;
 	    fs->border_color.lt = LT_NODRAW;
+	    set_border = TRUE;
 	    c_token++;
 	    continue;
 	}
 
-	/* We hit something unexpected */
-	if (!set_fill || set_param)
-	    break;
-	if (!(isanumber(c_token) || type_udv(c_token) == INTGR || type_udv(c_token) == CMPLX))
-	    break;
-
-	if (fs->fillstyle == FS_SOLID || fs->fillstyle == FS_TRANSPARENT_SOLID) {
-	    /* user sets 0...1, but is stored as an integer 0..100 */
-	    fs->filldensity = 100.0 * real_expression() + 0.5;
-	    if (fs->filldensity < 0)
-		fs->filldensity = 0;
-	    if (fs->filldensity > 100)
-		fs->filldensity = 100;
-	    set_param = TRUE;
-	} else if (fs->fillstyle == FS_PATTERN || fs->fillstyle == FS_TRANSPARENT_PATTERN) {
-	    fs->fillpattern = int_expression();
-	    if (fs->fillpattern < 0)
-		fs->fillpattern = 0;
-	    set_param = TRUE;
-	}
+	/* Keyword must belong to someone else */
+	break;
+    }
+    if (transparent) {
+	if (fs->fillstyle == FS_SOLID)
+	    fs->fillstyle = FS_TRANSPARENT_SOLID;
+        else if (fs->fillstyle == FS_PATTERN)
+	    fs->fillstyle = FS_TRANSPARENT_PATTERN;
     }
 }
 
@@ -1097,7 +1391,12 @@ parse_colorspec(struct t_colorspec *tc, int options)
 	c_token++;
 	tc->type = TC_LT;
 	tc->lt = LT_BACKGROUND;
+    } else if (equals(c_token,"black")) {
+	c_token++;
+	tc->type = TC_LT;
+	tc->lt = LT_BLACK;
     } else if (equals(c_token,"lt")) {
+	struct lp_style_type lptemp;
 	c_token++;
 	if (END_OF_COMMAND)
 	    int_error(c_token, "expected linetype");
@@ -1107,6 +1406,15 @@ parse_colorspec(struct t_colorspec *tc, int options)
 	    tc->type = TC_DEFAULT;
 	    int_warn(c_token,"illegal linetype");
 	}
+
+	/*
+	 * July 2014 - translate linetype into user-defined linetype color.
+	 * This is a CHANGE!
+	 * FIXME: calling load_linetype here may obviate the need to call it
+	 * many places in the higher level code.  They could be removed.
+	 */
+	load_linetype(&lptemp, tc->lt + 1);
+	*tc = lptemp.pm3d_color;
     } else if (options <= TC_LT) {
 	tc->type = TC_DEFAULT;
 	int_error(c_token, "only tc lt <n> possible here");
@@ -1178,14 +1486,17 @@ parse_color_name()
     if (almost_equals(c_token,"rgb$color") && almost_equals(c_token-1,"back$ground"))
 	c_token++;
     if ((string = try_to_get_string())) {
-	color = lookup_table_nth(pm3d_color_names_tbl, string);
-	if (color >= 0)
-	    color = pm3d_color_names_tbl[color].value;
-	else
-	    sscanf(string,"#%lx",&color);
+	int iret;
+	iret = lookup_table_nth(pm3d_color_names_tbl, string);
+	if (iret >= 0)
+	    color = pm3d_color_names_tbl[iret].value;
+	else if (string[0] == '#')
+	    iret = sscanf(string,"#%lx",&color);
+	else if (string[0] == '0' && (string[1] == 'x' || string[1] == 'X'))
+	    iret = sscanf(string,"%lx",&color);
 	free(string);
 	if (color == -2)
-	    int_error(c_token, "unrecognized color name and not a string \"#AARRGGBB\"");
+	    int_error(c_token, "unrecognized color name and not a string \"#AARRGGBB\" or \"0xAARRGGBB\"");
     } else {
 	color = int_expression();
     }
@@ -1274,25 +1585,32 @@ arrow_parse(
 	    continue;
 	}
 
+	if (almost_equals(c_token, "nobo$rder")) {
+	    if (set_headfilled++)
+		break;
+	    c_token++;
+	    arrow->headfill = AS_NOBORDER;
+	    continue;
+	}
 	if (almost_equals(c_token, "fill$ed")) {
 	    if (set_headfilled++)
 		break;
 	    c_token++;
-	    arrow->head_filled = 2;
+	    arrow->headfill = AS_FILLED;
 	    continue;
 	}
 	if (almost_equals(c_token, "empty")) {
 	    if (set_headfilled++)
 		break;
 	    c_token++;
-	    arrow->head_filled = 1;
+	    arrow->headfill = AS_EMPTY;
 	    continue;
 	}
 	if (almost_equals(c_token, "nofill$ed")) {
 	    if (set_headfilled++)
 		break;
 	    c_token++;
-	    arrow->head_filled = 0;
+	    arrow->headfill = AS_NOFILL;
 	    continue;
 	}
 
@@ -1313,6 +1631,15 @@ arrow_parse(
 	    /* invalid backangle --> default of 90.0 degrees */
 	    if (arrow->head_backangle <= arrow->head_angle)
 		arrow->head_backangle = 90.0;
+	    
+	    /* Assume adjustable size but check for 'fixed' instead */
+	    arrow->head_fixedsize = FALSE;
+	    continue;
+	}
+
+	if (almost_equals(c_token, "fix$ed")) {
+	    arrow->head_fixedsize = TRUE;
+	    c_token++;
 	    continue;
 	}
 
@@ -1320,21 +1647,21 @@ arrow_parse(
 	    if (set_layer++)
 		break;
 	    c_token++;
-	    arrow->layer = 0;
+	    arrow->layer = LAYER_BACK;
 	    continue;
 	}
 	if (equals(c_token, "front")) {
 	    if (set_layer++)
 		break;
 	    c_token++;
-	    arrow->layer = 1;
+	    arrow->layer = LAYER_FRONT;
 	    continue;
 	}
 
 	/* pick up a line spec - allow ls, but no point. */
 	{
 	    int stored_token = c_token;
-	    lp_parse(&arrow->lp_properties, TRUE, FALSE);
+	    lp_parse(&arrow->lp_properties, LP_ADHOC, FALSE);
 	    if (stored_token == c_token || set_line++)
 		break;
 	    continue;
