@@ -1,5 +1,5 @@
 /*
- * $Id: term_api.h,v 1.97.2.4 2012/10/09 16:30:15 sfeam Exp $
+ * $Id: term_api.h,v 1.134.2.6 2015/04/04 19:16:33 broeker Exp $
  */
 
 /* GNUPLOT - term_api.h */
@@ -44,20 +44,31 @@
 #include "gp_types.h"
 
 #include "color.h"
-#include "mousecmn.h"
 #include "tables.h"
+#ifdef OS2
+# include "mousecmn.h"
+#endif
 
 /* Constants that are interpreted by terminal driver routines */
 
 /* Default line type is LT_BLACK; reset to this after changing colors */
 #define LT_AXIS       (-1)
-#define LT_BLACK      (-2)
+#define LT_BLACK      (-2)		/* Base line type */
+#define LT_SOLID      (-2)		/* Synonym for base line type */
 #define LT_NODRAW     (-3)
 #define LT_BACKGROUND (-4)
 #define LT_UNDEFINED  (-5)
 #define LT_COLORFROMCOLUMN  (-6)	/* Used by hidden3d code */
 #define LT_DEFAULT    (-7)
-#define LT_SINGLECOLOR  (-8)		/* Used by hidden3d code */
+
+/* Pre-defined dash types */
+#define DASHTYPE_CUSTOM (-3)
+#define DASHTYPE_AXIS   (-2)
+#define DASHTYPE_SOLID  (-1)
+/* more...? */
+
+/* magic point type that indicates a character rather than a predefined symbol */
+#define PT_CHARACTER  (-9)
 
 /* Constant value passed to (term->text_angle)(ang) to generate vertical
  * text corresponding to old keyword "rotate", which produced the equivalent
@@ -84,20 +95,58 @@ typedef enum VERT_JUSTIFY {
     JUST_BOT
 } VERT_JUSTIFY;
 
+typedef enum t_linecap {
+    BUTT = 0,
+    ROUNDED,
+    SQUARE
+} t_linecap;
+
+/* custom dash pattern definition modeled after SVG terminal,
+ * but string specifications like "--.. " are also allowed and stored */
+#define DASHPATTERN_LENGTH 8
+
+typedef struct t_dashtype {
+	float pattern[DASHPATTERN_LENGTH];
+	char dstring[8];
+} t_dashtype;
+
+#define DEFAULT_DASHPATTERN {{0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f}, \
+                             {0,0,0,0,0,0,0,0} }
 
 typedef struct lp_style_type {	/* contains all Line and Point properties */
-    int     pointflag;		/* 0 if points not used, otherwise 1 */
+    int     flags;		/* e.g. LP_SHOW_POINTS */
     int     l_type;
     int	    p_type;
+    int     d_type;		/* Dashtype */
     int     p_interval;		/* Every Nth point in style LINESPOINTS */
     double  l_width;
     double  p_size;
-    TBOOLEAN use_palette;
+    unsigned long p_char;	/* char used if p_type = PT_CHARACTER */
     struct t_colorspec pm3d_color;
+    t_dashtype custom_dash_pattern;	/* per-line, user defined dashtype */
     /* ... more to come ? */
 } lp_style_type;
 
-#define DEFAULT_LP_STYLE_TYPE {0, LT_BLACK, 0, 0, 1.0, PTSZ_DEFAULT, FALSE, DEFAULT_COLORSPEC}
+#define DEFAULT_LP_STYLE_TYPE {0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 1.0, PTSZ_DEFAULT, 0, DEFAULT_COLORSPEC, DEFAULT_DASHPATTERN}
+
+/* Bit definitions for lp_style_type.flags */
+#define LP_SHOW_POINTS     (0x1) /* if not set, ignore the point properties of this line style */
+#define LP_NOT_INITIALIZED (0x2) /* internal flag used in set.c:parse_label_options */
+#define LP_EXPLICIT_COLOR  (0x4) /* set by lp_parse if the user provided a color spec */
+
+#define DEFAULT_COLOR_SEQUENCE { 0x9400d3, 0x009e73, 0x56b4e9, 0xe69f00, \
+                                 0xf0e442, 0x0072b2, 0xe51e10, 0x000000 }
+#define PODO_COLOR_SEQUENCE { 0x000000, 0xe69f00, 0x56b4e9, 0x009e73, \
+                              0xf0e442, 0x0072b2, 0xd55e00, 0xcc79a7 }
+
+#define DEFAULT_MONO_LINETYPES { \
+	{0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 1.0 /*linewidth*/, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BLACK, 0, 1 /* dt 2 */, 0, 1.0 /*linewidth*/, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BLACK, 0, 2 /* dt 3 */, 0, 1.0 /*linewidth*/, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BLACK, 0, 3 /* dt 4 */, 0, 1.0 /*linewidth*/, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BLACK, 0, 0 /* dt 1 */, 0, 2.0 /*linewidth*/, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BLACK, 0, DASHTYPE_CUSTOM, 0, 1.2 /*linewidth*/, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, {{16.,8.,2.,5.,2.,5.,2.,8.},{0,0,0,0,0,0,0,0}}} \
+}
 
 typedef enum e_arrow_head {
 	NOHEAD = 0,
@@ -108,6 +157,13 @@ typedef enum e_arrow_head {
 
 extern const char *arrow_head_names[4];
 
+typedef enum arrowheadfill {
+	AS_NOFILL = 0,
+	AS_EMPTY,
+	AS_FILLED,
+	AS_NOBORDER
+} arrowheadfill;
+
 typedef struct arrow_style_type {    /* contains all Arrow properties */
     int tag;                         /* -1 (local), AS_VARIABLE, or style index */
     int layer;	                     /* 0 = back, 1 = front */
@@ -119,7 +175,8 @@ typedef struct arrow_style_type {    /* contains all Arrow properties */
     int head_lengthunit;             /* unit (x1, x2, screen, graph) */
     double head_angle;               /* front angle / deg */
     double head_backangle;           /* back angle / deg */
-    unsigned int head_filled;        /* filled heads: 0=not, 1=empty, 2=filled */
+    arrowheadfill headfill;	     /* AS_FILLED etc */
+    TBOOLEAN head_fixedsize;         /* Adapt the head size for short arrow shafts? */
     /* ... more to come ? */
 } arrow_style_type;
 
@@ -138,9 +195,19 @@ typedef enum termlayer {
 	TERM_LAYER_RESET_PLOTNO,
 	TERM_LAYER_BEFORE_ZOOM,
 	TERM_LAYER_BEGIN_PM3D_MAP,
-	TERM_LAYER_END_PM3D_MAP
+	TERM_LAYER_END_PM3D_MAP,
+	TERM_LAYER_BEGIN_IMAGE,
+	TERM_LAYER_END_IMAGE
 } t_termlayer;
 
+/* Options used by the terminal entry point term->waitforinput(). */
+#define TERM_ONLY_CHECK_MOUSING	1
+#define TERM_EVENT_POLL_TIMEOUT 0	/* select() timeout in usec */
+
+/* Options used by the terminal entry point term->hypertext(). */
+#define TERM_HYPERTEXT_TOOLTIP 0
+#define TERM_HYPERTEXT_TITLE   1
+
 typedef struct fill_style_type {
     int fillstyle;
     int filldensity;
@@ -148,7 +215,19 @@ typedef struct fill_style_type {
     t_colorspec border_color;
 } fill_style_type;
 
-typedef enum t_fillstyle { FS_EMPTY, FS_SOLID, FS_PATTERN, FS_DEFAULT, 
+#ifdef EAM_BOXED_TEXT
+/* Options used by the terminal entry point term->boxed_text() */
+typedef enum t_textbox_options {
+	TEXTBOX_INIT = 0,
+	TEXTBOX_OUTLINE,
+	TEXTBOX_BACKGROUNDFILL,
+	TEXTBOX_MARGINS,
+	TEXTBOX_FINISH,
+	TEXTBOX_GREY
+} t_textbox_options;
+#endif
+
+typedef enum t_fillstyle { FS_EMPTY, FS_SOLID, FS_PATTERN, FS_DEFAULT,
 			   FS_TRANSPARENT_SOLID, FS_TRANSPARENT_PATTERN }
 	     t_fillstyle;
 #define FS_OPAQUE (FS_SOLID + (100<<4))
@@ -162,22 +241,29 @@ typedef struct t_image {
     TBOOLEAN fallback; /* true == don't use terminal-specific code */
 } t_image;
 
+/* Operations possible with term->modify_plots() */
+#define MODPLOTS_SET_VISIBLE         (1<<0)
+#define MODPLOTS_SET_INVISIBLE       (1<<1)
+#define MODPLOTS_INVERT_VISIBILITIES (MODPLOTS_SET_VISIBLE|MODPLOTS_SET_INVISIBLE)
+
 /* Values for the flags field of TERMENTRY
  */
-#define TERM_CAN_MULTIPLOT    1  /* tested if stdout not redirected */
-#define TERM_CANNOT_MULTIPLOT 2  /* tested if stdout is redirected  */
-#define TERM_BINARY           4  /* open output file with "b"       */
-#define TERM_INIT_ON_REPLOT   8  /* call term->init() on replot     */
-#define TERM_IS_POSTSCRIPT   16  /* post, next, pslatex, etc        */
-#define TERM_ENHANCED_TEXT   32  /* enhanced text mode is enabled   */
-#define TERM_NO_OUTPUTFILE   64  /* terminal doesnt write to a file */
-#define TERM_CAN_CLIP       128  /* terminal does its own clipping  */
-#define TERM_CAN_DASH       256  /* terminal supports dashed lines  */
-#define TERM_ALPHA_CHANNEL  512  /* alpha channel transparency      */
-#define TERM_MONOCHROME    1024  /* term is running in mono mode    */
-#define TERM_LINEWIDTH     2048  /* support for set term linewidth  */
-#define TERM_FONTSCALE     4096  /* terminal supports fontscale     */
-#define TERM_IS_LATEX      8192  /* text uses TeX markup            */
+#define TERM_CAN_MULTIPLOT    (1<<0)	/* tested if stdout not redirected */
+#define TERM_CANNOT_MULTIPLOT (1<<1)	/* tested if stdout is redirected  */
+#define TERM_BINARY           (1<<2)	/* open output file with "b"       */
+#define TERM_INIT_ON_REPLOT   (1<<3)	/* call term->init() on replot     */
+#define TERM_IS_POSTSCRIPT    (1<<4)	/* post, next, pslatex, etc        */
+#define TERM_ENHANCED_TEXT    (1<<5)	/* enhanced text mode is enabled   */
+#define TERM_NO_OUTPUTFILE    (1<<6)	/* terminal doesnt write to a file */
+#define TERM_CAN_CLIP         (1<<7)	/* terminal does its own clipping  */
+#define TERM_CAN_DASH         (1<<8)	/* terminal supports dashed lines  */
+#define TERM_ALPHA_CHANNEL    (1<<9)	/* alpha channel transparency      */
+#define TERM_MONOCHROME      (1<<10)	/* term is running in mono mode    */
+#define TERM_LINEWIDTH       (1<<11)	/* support for set term linewidth  */
+#define TERM_FONTSCALE       (1<<12)	/* terminal supports fontscale     */
+#define TERM_IS_LATEX        (1<<13)	/* text uses TeX markup            */
+#define TERM_EXTENDED_COLOR  (1<<14)	/* uses EXTENDED_COLOR_SPECS       */
+#define TERM_NULL_SET_COLOR  (1<<15)	/* no support for RGB color        */
 
 /* The terminal interface structure --- heart of the terminal layer.
  *
@@ -216,7 +302,7 @@ typedef struct TERMENTRY {
     void (*fillbox) __PROTO((int, unsigned int, unsigned int, unsigned int, unsigned int)); /* clear in multiplot mode */
     void (*linewidth) __PROTO((double linewidth));
 #ifdef USE_MOUSE
-    int (*waitforinput) __PROTO((void));     /* used for mouse input */
+    int (*waitforinput) __PROTO((int));     /* used for mouse and hotkey input */
     void (*put_tmptext) __PROTO((int, const char []));   /* draws temporary text; int determines where: 0=statusline, 1,2: at corners of zoom box, with \r separating text above and below the point */
     void (*set_ruler) __PROTO((int, int));    /* set ruler location; x<0 switches ruler off */
     void (*set_cursor) __PROTO((int, int, int));   /* set cursor style and corner of rubber band */
@@ -261,7 +347,7 @@ typedef struct TERMENTRY {
  */
     void (*layer) __PROTO((t_termlayer));
 
-/* Begin/End path control. 
+/* Begin/End path control.
  * Needed by PostScript-like devices in order to join the endpoints of
  * a polygon cleanly.
  */
@@ -272,14 +358,25 @@ typedef struct TERMENTRY {
  */
     double tscale;
 
+/* Pass hypertext for inclusion in the output plot */
+    void (*hypertext) __PROTO((int type, const char *text));
+
+#ifdef EAM_BOXED_TEXT
+    void (*boxed_text) __PROTO((unsigned int, unsigned int, int));
+#endif
+
+    void (*modify_plots) __PROTO((unsigned int operations, int plotno));
+
+    void (*dashtype) __PROTO((int type, t_dashtype *custom_dash_pattern));
+
 } TERMENTRY;
 
 # define termentry TERMENTRY
 
 enum set_encoding_id {
    S_ENC_DEFAULT, S_ENC_ISO8859_1, S_ENC_ISO8859_2, S_ENC_ISO8859_9, S_ENC_ISO8859_15,
-   S_ENC_CP437, S_ENC_CP850, S_ENC_CP852, S_ENC_CP950, 
-   S_ENC_CP1250, S_ENC_CP1251, S_ENC_CP1254, 
+   S_ENC_CP437, S_ENC_CP850, S_ENC_CP852, S_ENC_CP950,
+   S_ENC_CP1250, S_ENC_CP1251, S_ENC_CP1252, S_ENC_CP1254,
    S_ENC_KOI8_R, S_ENC_KOI8_U, S_ENC_SJIS,
    S_ENC_UTF8,
    S_ENC_INVALID
@@ -301,21 +398,6 @@ enum set_encoding_id {
  * semaphores => the code in gpexecute.c is used, and nothing more from here.
  */
 
-#ifdef PIPE_IPC
-
-enum { IPC_BACK_UNUSABLE = -2, IPC_BACK_CLOSED = -1 };
-
-/*
- * Currently only used for PIPE_IPC, but in principle
- * every term could use this file descriptor to write back
- * commands to gnuplot.  Note, that terminals using this fd
- * should set it to a negative value when closing. (joze)
- */
-/* HBB 20020225: currently not used anywhere outside term.c --> make
- * it static */
-/* extern int ipc_back_fd; */
-
-# endif /* PIPE_IPC */
 
 /* options handling */
 enum { UNSET = -1, no = 0, yes = 1 };
@@ -331,11 +413,12 @@ extern int curr_arrow_headlength;
 /* angle in degrees */
 extern double curr_arrow_headangle;
 extern double curr_arrow_headbackangle;
-/* arrow head filled or not */
-extern int curr_arrow_headfilled;
+extern arrowheadfill curr_arrow_headfilled;
+extern TBOOLEAN curr_arrow_headfixedsize;
 
 /* Recycle count for user-defined linetypes */
 extern int linetype_recycle_count;
+extern int mono_recycle_count;
 
 /* Current 'output' file: name and open filehandle */
 extern char *outstr;
@@ -353,6 +436,7 @@ extern FILE *gpoutfile;
 extern FILE *gppsfile;
 extern char *PS_psdir;
 
+extern TBOOLEAN monochrome;
 extern TBOOLEAN multiplot;
 
 /* 'set encoding' support: index of current encoding ... */
@@ -394,6 +478,7 @@ struct termentry *change_term __PROTO((const char *name, int length));
 
 void write_multiline __PROTO((unsigned int, unsigned int, char *, JUSTIFY, VERT_JUSTIFY, int, const char *));
 int estimate_strlen __PROTO((char *));
+char *estimate_plaintext __PROTO((char *));
 void list_terms __PROTO((void));
 char* get_terminals_names __PROTO((void));
 struct termentry *set_term __PROTO((void));
@@ -401,7 +486,7 @@ void init_terminal __PROTO((void));
 void test_term __PROTO((void));
 
 /* Support for enhanced text mode. */
-char *enhanced_recursion __PROTO((char *p, TBOOLEAN brace,
+const char *enhanced_recursion __PROTO((const char *p, TBOOLEAN brace,
                                          char *fontname, double fontsize,
                                          double base, TBOOLEAN widthflag,
                                          TBOOLEAN showflag, int overprint));
@@ -421,7 +506,8 @@ int style_from_fill __PROTO((struct fill_style_type *));
 #ifdef EAM_OBJECTS
 /* Terminal-independent routine to draw a circle or arc */
 void do_arc __PROTO(( unsigned int cx, unsigned int cy, double radius,
-                      double arc_start, double arc_end, int style));
+                      double arc_start, double arc_end,
+		      int style, TBOOLEAN wedge));
 #endif
 
 #ifdef LINUXVGA
@@ -445,6 +531,7 @@ void PM_set_gpPMmenu __PROTO((struct t_gpPMmenu * gpPMmenu));
 # endif
 #endif
 
+int load_dashtype __PROTO((struct t_dashtype *dt, int tag));
 void lp_use_properties __PROTO((struct lp_style_type *lp, int tag));
 void load_linetype __PROTO((struct lp_style_type *lp, int tag));
 
@@ -452,4 +539,9 @@ void load_linetype __PROTO((struct lp_style_type *lp, int tag));
 void newpath __PROTO((void));
 void closepath __PROTO((void));
 
+/* Generic wrapper to check for mouse events or hotkeys during
+ * non-interactive input (e.g. "load")
+ */
+void check_for_mouse_events __PROTO((void));
+
 #endif /* GNUPLOT_TERM_API_H */
