@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: unset.c,v 1.149.2.5 2014/02/26 07:36:18 markisch Exp $"); }
+static char *RCSid() { return RCSid("$Id: unset.c,v 1.206.2.6 2015/03/29 18:08:08 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - unset.c */
@@ -41,7 +41,6 @@ static char *RCSid() { return RCSid("$Id: unset.c,v 1.149.2.5 2014/02/26 07:36:1
 #include "contour.h"
 #include "datafile.h"
 #include "fit.h"
-#include "gadgets.h"
 #include "gp_hist.h"
 #include "hidden3d.h"
 #include "misc.h"
@@ -50,6 +49,7 @@ static char *RCSid() { return RCSid("$Id: unset.c,v 1.149.2.5 2014/02/26 07:36:1
 #include "plot2d.h"
 #include "plot3d.h"
 #include "tables.h"
+#include "tabulate.h"
 #include "term_api.h"
 #include "util.h"
 #include "variable.h"
@@ -66,23 +66,25 @@ static void unset_border __PROTO((void));
 static void unset_boxplot __PROTO((void));
 static void unset_boxwidth __PROTO((void));
 static void unset_fillstyle __PROTO((void));
-static void unset_clabel __PROTO((void));
 static void unset_clip __PROTO((void));
 static void unset_cntrparam __PROTO((void));
+static void unset_cntrlabel __PROTO((void));
 static void unset_contour __PROTO((void));
+static void unset_dashtype __PROTO((void));
 static void unset_dgrid3d __PROTO((void));
 static void unset_dummy __PROTO((void));
 static void unset_encoding __PROTO((void));
 static void unset_decimalsign __PROTO((void));
 static void unset_fit __PROTO((void));
-static void unset_format __PROTO((void));
 static void unset_grid __PROTO((void));
 static void unset_hidden3d __PROTO((void));
 static void unset_histogram __PROTO((void));
+#ifdef EAM_BOXED_TEXT
+static void unset_textbox_style __PROTO((void));
+#endif
 static void unset_historysize __PROTO((void));
 static void unset_isosamples __PROTO((void));
 static void unset_key __PROTO((void));
-static void unset_keytitle __PROTO((void));
 static void unset_label __PROTO((void));
 static void delete_label __PROTO((struct text_label * prev, struct text_label * this));
 static void unset_linestyle __PROTO((struct linestyle_def **head));
@@ -98,9 +100,6 @@ static void unset_loadpath __PROTO((void));
 static void unset_locale __PROTO((void));
 static void reset_logscale __PROTO((AXIS_INDEX));
 static void unset_logscale __PROTO((void));
-#ifdef GP_MACROS
-static void unset_macros __PROTO((void));
-#endif
 static void unset_mapping __PROTO((void));
 static void unset_margin __PROTO((t_position *));
 static void unset_missing __PROTO((void));
@@ -131,7 +130,6 @@ static void unset_surface __PROTO((void));
 static void unset_table __PROTO((void));
 static void unset_terminal __PROTO((void));
 static void unset_tics __PROTO((AXIS_INDEX));
-static void unset_ticscale __PROTO((void));
 static void unset_ticslevel __PROTO((void));
 static void unset_timefmt __PROTO((void));
 static void unset_timestamp __PROTO((void));
@@ -151,6 +149,7 @@ unset_command()
 {
     int found_token;
     int save_token;
+    int i;
 
     c_token++;
 
@@ -184,18 +183,24 @@ unset_command()
     case S_BOXWIDTH:
 	unset_boxwidth();
 	break;
-    case S_CLABEL:
-	unset_clabel();
-	break;
     case S_CLIP:
 	unset_clip();
 	break;
     case S_CNTRPARAM:
 	unset_cntrparam();
 	break;
+    case S_CNTRLABEL:
+	unset_cntrlabel();
+	break;
+    case S_CLABEL:	/* deprecated command */
+	clabel_onecolor = TRUE;
+	break;
     case S_CONTOUR:
 	unset_contour();
 	break;
+    case S_DASHTYPE:
+	unset_dashtype();
+	break;
     case S_DGRID3D:
 	unset_dgrid3d();
 	break;
@@ -212,7 +217,8 @@ unset_command()
 	unset_fit();
 	break;
     case S_FORMAT:
-	unset_format();
+	c_token--;
+	set_format();
 	break;
     case S_GRID:
 	unset_grid();
@@ -220,7 +226,9 @@ unset_command()
     case S_HIDDEN3D:
 	unset_hidden3d();
 	break;
-    case S_HISTORYSIZE:
+    case S_HISTORY:
+	break; /* FIXME: reset to default values? */
+    case S_HISTORYSIZE:	/* Deprecated */
 	unset_historysize();
 	break;
     case S_ISOSAMPLES:
@@ -229,15 +237,16 @@ unset_command()
     case S_KEY:
 	unset_key();
 	break;
-    case S_KEYTITLE:
-	unset_keytitle();
-	break;
     case S_LABEL:
 	unset_label();
 	break;
     case S_LINETYPE:
 	unset_linetype();
 	break;
+    case S_LINK:
+	c_token--;
+	link_command();
+	break;
     case S_LOADPATH:
 	unset_loadpath();
 	break;
@@ -247,14 +256,18 @@ unset_command()
     case S_LOGSCALE:
 	unset_logscale();
 	break;
-#ifdef GP_MACROS
     case S_MACROS:
-	unset_macros();
+	/* Aug 2013 - macros are always enabled */
 	break;
-#endif
     case S_MAPPING:
 	unset_mapping();
 	break;
+    case S_MARGIN:
+	unset_margin(&lmargin);
+	unset_margin(&rmargin);
+	unset_margin(&tmargin);
+	unset_margin(&bmargin);
+	break;
     case S_BMARGIN:
 	unset_margin(&bmargin);
 	break;
@@ -272,14 +285,40 @@ unset_command()
 	    df_fortran_constants = FALSE;
 	    c_token++;
 	    break;
+	} else if (almost_equals(c_token,"miss$ing")) {
+	    unset_missing();
+	    c_token++;
+	    break;
+	} else if (almost_equals(c_token,"sep$arators")) {
+	    free(df_separators);
+	    df_separators = NULL;
+	    c_token++;
+	    break;
+	} else if (almost_equals(c_token,"com$mentschars")) {
+	    free(df_commentschars);
+	    df_commentschars = gp_strdup(DEFAULT_COMMENTS_CHARS);
+	    c_token++;
+	    break;
+	} else if (almost_equals(c_token,"bin$ary")) {
+	    df_unset_datafile_binary();
+	    c_token++;
+	    break;
+	} else if (almost_equals(c_token,"nofpe_trap")) {
+	    df_nofpe_trap = FALSE;
+	    c_token++;
+	    break;
 	}
 	df_fortran_constants = FALSE;
 	unset_missing();
-	df_separator = '\0';
+	free(df_separators);
+	df_separators = NULL;
 	free(df_commentschars);
 	df_commentschars = gp_strdup(DEFAULT_COMMENTS_CHARS);
 	df_unset_datafile_binary();
 	break;
+    case S_MONOCHROME:
+	unset_monochrome();
+	break;
 #ifdef USE_MOUSE
     case S_MOUSE:
 	unset_mouse();
@@ -332,6 +371,15 @@ unset_command()
     case S_RTICS:
 	unset_tics(POLAR_AXIS);
 	break;
+    case S_PAXIS:
+	i = int_expression();
+	if (i <= 0 || i > MAX_PARALLEL_AXES)
+	    int_error(c_token, "expecting parallel axis number");
+	if (almost_equals(c_token, "tic$s")) {
+	    unset_tics(PARALLEL_AXES+i-1);
+	    c_token++;
+	}
+	break;
     case S_SAMPLES:
 	unset_samples();
 	break;
@@ -351,10 +399,10 @@ unset_command()
 	unset_terminal();
 	break;
     case S_TICS:
-	unset_tics(AXIS_ARRAY_SIZE);
+	unset_tics(ALL_AXES);
 	break;
     case S_TICSCALE:
-	unset_ticscale();
+	int_warn(c_token, "Deprecated syntax - use 'set tics scale default'");
 	break;
     case S_TICSLEVEL:
     case S_XYPLANE:
@@ -436,6 +484,9 @@ unset_command()
     case S_CBMTICS:
 	unset_month_day_tics(FIRST_X_AXIS);
 	break;
+    case S_MRTICS:
+	unset_minitics(POLAR_AXIS);
+	break;
     case S_XDATA:
 	unset_timedata(FIRST_X_AXIS);
 	break;
@@ -536,7 +587,7 @@ unset_command()
     }
 
     update_gpval_variables(0);
-    
+
     set_iterator = cleanup_iteration(set_iterator);
 }
 
@@ -574,7 +625,6 @@ unset_arrow()
 		return;		/* exit, our job is done */
 	    }
 	}
-	int_error(c_token, "arrow not found");
     }
 }
 
@@ -685,15 +735,6 @@ unset_fillstyle()
 }
 
 
-/* process 'unset clabel' command */
-static void
-unset_clabel()
-{
-    /* FIXME? reset_command() uses TRUE */
-    label_contours = FALSE;
-}
-
-
 /* process 'unset clip' command */
 static void
 unset_clip()
@@ -726,6 +767,18 @@ unset_cntrparam()
     contour_levels_kind = LEVELS_AUTO;
 }
 
+/* process 'unset cntrlabel' command */
+static void
+unset_cntrlabel()
+{
+    clabel_onecolor = FALSE;
+    clabel_start = 5;
+    clabel_interval = 20;
+    strcpy(contour_format, "%8.3g");
+    free(clabel_font);
+    clabel_font = NULL;
+}
+
 
 /* process 'unset contour' command */
 static void
@@ -735,6 +788,29 @@ unset_contour()
 }
 
 
+/* process 'unset dashtype' command */
+static void
+unset_dashtype()
+{
+    struct custom_dashtype_def *this, *prev;
+    if (END_OF_COMMAND) {
+	/* delete all */
+	while (first_custom_dashtype != NULL)
+	    delete_dashtype((struct custom_dashtype_def *) NULL, first_custom_dashtype);
+	}
+    else {		
+	int tag = int_expression();
+	for (this = first_custom_dashtype, prev = NULL; this != NULL;
+	 prev = this, this = this->next) {
+	    if (this->tag == tag) {
+		delete_dashtype(prev, this);
+		break;
+	    }
+	}
+    }
+}
+
+
 /* process 'unset dgrid3d' command */
 static void
 unset_dgrid3d()
@@ -753,8 +829,11 @@ unset_dgrid3d()
 static void
 unset_dummy()
 {
+    int i;
     strcpy(set_dummy_var[0], "x");
     strcpy(set_dummy_var[1], "y");
+    for (i=2; i<MAX_NUM_VAR; i++)
+	*set_dummy_var[i] = '\0';
 }
 
 
@@ -782,47 +861,24 @@ unset_decimalsign()
 static void
 unset_fit()
 {
-    if (fitlogfile != NULL)
-	free(fitlogfile);
+    free(fitlogfile);
     fitlogfile = NULL;
-    fit_errorvariables = FALSE;
-    fit_prescale = FALSE;
+    fit_errorvariables = TRUE;
+    fit_covarvariables = FALSE;
+    fit_errorscaling = TRUE;
+    fit_prescale = TRUE;
+    fit_verbosity = BRIEF;
+    del_udv_by_name((char *)FITLIMIT, FALSE);
+    epsilon_abs = 0.;
+    del_udv_by_name((char *)FITMAXITER, FALSE);
+    del_udv_by_name((char *)FITSTARTLAMBDA, FALSE);
+    del_udv_by_name((char *)FITLAMBDAFACTOR, FALSE);
+    free(fit_script);
+    fit_script = NULL;
+    fit_wrap = 0;
+    /* do not reset fit_v4compatible */
 }
 
-
-/* process 'unset format' command */
-/* FIXME: compare and merge with set.c::set_format */
-static void
-unset_format()
-{
-    TBOOLEAN set_for_axis[AXIS_ARRAY_SIZE] = AXIS_ARRAY_INITIALIZER(FALSE);
-    int axis;
-
-    if ((axis = lookup_table(axisname_tbl, c_token)) >= 0) {
-	set_for_axis[axis] = TRUE;
-    } else if (equals(c_token,"xy") || equals(c_token,"yx")) {
-	set_for_axis[FIRST_X_AXIS]
-	    = set_for_axis[FIRST_Y_AXIS]
-	    = TRUE;
-	c_token++;
-    } else if (isstring(c_token) || END_OF_COMMAND) {
-	/* Assume he wants all */
-	for (axis = 0; axis < AXIS_ARRAY_SIZE; axis++)
-	    set_for_axis[axis] = TRUE;
-    }
-
-    if (END_OF_COMMAND) {
-	SET_DEFFORMAT(FIRST_X_AXIS , set_for_axis);
-	SET_DEFFORMAT(FIRST_Y_AXIS , set_for_axis);
-	SET_DEFFORMAT(FIRST_Z_AXIS , set_for_axis);
-	SET_DEFFORMAT(SECOND_X_AXIS, set_for_axis);
-	SET_DEFFORMAT(SECOND_Y_AXIS, set_for_axis);
-	SET_DEFFORMAT(COLOR_AXIS   , set_for_axis);
-	SET_DEFFORMAT(POLAR_AXIS   , set_for_axis);
-    }
-}
-
-
 /* process 'unset grid' command */
 static void
 unset_grid()
@@ -850,19 +906,26 @@ unset_hidden3d()
 static void
 unset_histogram()
 {
-    histogram_opts.type = HT_CLUSTERED;
-    histogram_opts.gap = 2;
+    histogram_style foo = DEFAULT_HISTOGRAM_STYLE;
+    free(histogram_opts.title.font);
+    free_histlist(&histogram_opts);
+    histogram_opts = foo;
 }
 
-/* process 'unset historysize' command */
+#ifdef EAM_BOXED_TEXT
+static void
+unset_textbox_style()
+{
+    textbox_style foo = DEFAULT_TEXTBOX_STYLE;
+    textbox_opts = foo;
+}
+#endif
+
+/* process 'unset historysize' command DEPRECATED */
 static void
 unset_historysize()
 {
-#ifdef GNUPLOT_HISTORY
     gnuplot_history_size = -1; /* don't ever truncate the history. */
-#else
-    int_error(c_token, "Command 'unset historysize' requires history support.");
-#endif
 }
 
 
@@ -885,6 +948,9 @@ void
 reset_key()
 {
     legend_key temp_key = DEFAULT_KEY_PROPS;
+    free(keyT.font);
+    free(keyT.title.text);
+    free(keyT.title.font);
     memcpy(&keyT, &temp_key, sizeof(keyT));
 }
 
@@ -897,15 +963,6 @@ unset_key()
 }
 
 
-/* process 'unset keytitle' command */
-static void
-unset_keytitle()
-{
-    legend_key *key = &keyT;
-    key->title[0] = '\0';	/* empty string */
-}
-
-
 /* process 'unset label' command */
 static void
 unset_label()
@@ -931,7 +988,6 @@ unset_label()
 		return;		/* exit, our job is done */
 	    }
 	}
-	/* int_warn(c_token, "label not found"); */
     }
 }
 
@@ -961,7 +1017,7 @@ unset_linestyle(struct linestyle_def **head)
 {
     int tag = int_expression();
     struct linestyle_def *this, *prev;
-    for (this = *head, prev = NULL; this != NULL; 
+    for (this = *head, prev = NULL; this != NULL;
 	 prev = this, this = this->next) {
 	if (this->tag == tag) {
 	    delete_linestyle(head, prev, this);
@@ -982,7 +1038,6 @@ unset_linetype()
 }
 
 #ifdef EAM_OBJECTS
-/* process 'unset rectangle' command */
 static void
 unset_object()
 {
@@ -1007,9 +1062,6 @@ unset_object()
 		return;		/* exit, our job is done */
 	    }
 	}
-	/*
-	int_warn(c_token, "object %d not found", tag);
-	 */
     }
 }
 
@@ -1057,7 +1109,10 @@ reset_logscale(AXIS_INDEX axis)
 {
     TBOOLEAN undo_rlog = (axis == POLAR_AXIS && R_AXIS.log);
     axis_array[axis].log = FALSE;
+    /* Do not zero the base because we can still use it for gprintf formats
+     * %L and %l with linear axis scaling.
     axis_array[axis].base = 0.0;
+     */
     if (undo_rlog)
 	rrange_to_xy();
 }
@@ -1091,22 +1146,11 @@ unset_logscale()
 	++c_token;
     }
 
-#ifdef VOLATILE_REFRESH
     /* Because the log scaling is applied during data input, a quick refresh */
     /* using existing stored data will not work if the log setting changes.  */
-    refresh_ok = 0;
-#endif
+    SET_REFRESH_OK(E_REFRESH_NOT_OK, 0);
 }
 
-#ifdef GP_MACROS
-/* process 'unset macros' command */
-static void
-unset_macros()
-{
-    expand_macros = FALSE;
-}
-#endif
-
 /* process 'unset mapping3d' command */
 static void
 unset_mapping()
@@ -1162,11 +1206,11 @@ unset_tics(AXIS_INDEX axis)
     unsigned int iend = AXIS_ARRAY_SIZE;
     unsigned int i;
 
-    if (axis < AXIS_ARRAY_SIZE) {
+    if (axis != ALL_AXES) {
 	istart = axis;
 	iend = axis + 1;
     }
-	
+
     for (i = istart; i < iend; ++i) {
 	axis_array[i].ticmode = NO_TICS;
 
@@ -1178,7 +1222,8 @@ unset_tics(AXIS_INDEX axis)
 	axis_array[i].ticdef.textcolor.lt = 0;
 	axis_array[i].ticdef.textcolor.value = 0;
 	axis_array[i].ticdef.offset = tics_nooffset;
-	axis_array[i].ticdef.rangelimited = FALSE;
+	axis_array[i].ticdef.rangelimited = (i >= PARALLEL_AXES) ? TRUE : FALSE;
+	axis_array[i].ticdef.enhanced = TRUE;
 	axis_array[i].tic_rotate = 0;
 	axis_array[i].ticscale = 1.0;
 	axis_array[i].miniticscale = 0.5;
@@ -1196,6 +1241,17 @@ unset_month_day_tics(AXIS_INDEX axis)
     axis_array[axis].ticdef.type = TIC_COMPUTED;
 }
 
+void
+unset_monochrome()
+{
+    monochrome = FALSE;
+    if (equals(c_token,"lt") || almost_equals(c_token,"linet$ype")) {
+	c_token++;
+	if (!END_OF_COMMAND)
+	    unset_linestyle(&first_mono_linestyle);
+    }
+}
+
 /* process 'unset offsets' command */
 static void
 unset_offsets()
@@ -1235,7 +1291,7 @@ unset_output()
 static void
 unset_print()
 {
-    print_set_output(NULL, FALSE);
+    print_set_output(NULL, FALSE, FALSE);
 }
 
 /* process 'unset psdir' command */
@@ -1295,6 +1351,7 @@ unset_pm3d()
     /* reset styles, required to 'plot something' after e.g. 'set pm3d map' */
     if (data_style == PM3DSURFACE) data_style = POINTSTYLE;
     if (func_style == PM3DSURFACE) func_style = LINES;
+    pm3d.border.l_type = LT_NODRAW;
 }
 
 
@@ -1377,6 +1434,9 @@ unset_style()
 #endif
 	unset_histogram();
 	unset_boxplot();
+#ifdef EAM_BOXED_TEXT
+	unset_textbox_style();
+#endif
 	c_token++;
 	return;
     }
@@ -1424,13 +1484,19 @@ unset_style()
 	unset_style_ellipse();
 	c_token++;
 	break;
+#endif
+#ifdef EAM_BOXED_TEXT
+    case SHOW_STYLE_TEXTBOX:
+	unset_textbox_style();
+	c_token++;
+	break;
 #endif
     case SHOW_STYLE_BOXPLOT:
 	unset_boxplot();
 	c_token++;
 	break;
     default:
-	int_error(c_token, "unrecognized style option");
+	int_error(c_token, "unrecognized style");
     }
 }
 
@@ -1450,6 +1516,7 @@ unset_table()
     if (table_outfile)
 	fclose(table_outfile);
     table_outfile = NULL;
+    table_var = NULL;
     table_mode = FALSE;
 }
 
@@ -1474,22 +1541,6 @@ unset_terminal()
 }
 
 
-/* process 'unset ticscale' command */
-static void
-unset_ticscale()
-{
-    unsigned int i;
-
-    int_warn(c_token,
-	     "Deprecated syntax - please use 'set tics scale default'");
-
-    for (i = 0; i < AXIS_ARRAY_SIZE; ++i) {
-	axis_array[i].ticscale = 1.0;
-	axis_array[i].miniticscale = 0.5;
-    }
-}
-
-
 /* process 'unset ticslevel' command */
 static void
 unset_ticslevel()
@@ -1503,19 +1554,8 @@ unset_ticslevel()
 static void
 unset_timefmt()
 {
-    int axis;
-
-    if (END_OF_COMMAND)
-	for (axis=0; axis < AXIS_ARRAY_SIZE; axis++)
-	    strcpy(axis_array[axis].timefmt,TIMEFMT);
-    else if ((axis=lookup_table(axisname_tbl, c_token)) >= 0) {
-	strcpy(axis_array[axis].timefmt, TIMEFMT);
-	c_token++;
-    }
-    else
-	/* int_error() from inside 'reset' would cause problems */
-	int_warn(c_token, "expected optional axis name");
-
+    free(timefmt);
+    timefmt = gp_strdup(TIMEFMT);
 }
 
 
@@ -1533,7 +1573,6 @@ unset_timestamp()
 static void
 unset_view()
 {
-    splot_map_deactivate();
     splot_map = FALSE;
     aspect_ratio_3D = 0;
     surface_rot_z = 30.0;
@@ -1556,6 +1595,7 @@ static void
 unset_timedata(AXIS_INDEX axis)
 {
     axis_array[axis].datatype = DT_NORMAL;
+    axis_array[axis].tictype = DT_NORMAL;
 }
 
 
@@ -1563,8 +1603,13 @@ unset_timedata(AXIS_INDEX axis)
 static void
 unset_range(AXIS_INDEX axis)
 {
-    /* FIXME HBB 20000506: do we want to reset the axis autoscale and
-     * min/max, too?  */
+    axis_array[axis].set_autoscale = AUTOSCALE_BOTH;
+    axis_array[axis].writeback_min = axis_array[axis].set_min
+	= axis_defaults[GPMIN(axis,PARALLEL_AXES)].min;
+    axis_array[axis].writeback_max = axis_array[axis].set_max
+	= axis_defaults[GPMIN(axis,PARALLEL_AXES)].max;
+    axis_array[axis].min_constraint = CONSTRAINT_NONE;
+    axis_array[axis].max_constraint = CONSTRAINT_NONE;
     axis_array[axis].range_flags = 0;
 }
 
@@ -1572,7 +1617,9 @@ unset_range(AXIS_INDEX axis)
 static void
 unset_zeroaxis(AXIS_INDEX axis)
 {
-    axis_array[axis].zeroaxis = default_axis_zeroaxis;
+    if (axis_array[axis].zeroaxis != &default_axis_zeroaxis)
+	free(axis_array[axis].zeroaxis);
+    axis_array[axis].zeroaxis = NULL;
 }
 
 
@@ -1591,7 +1638,7 @@ unset_all_zeroaxes()
 static void
 unset_axislabel_or_title(text_label *label)
 {
-    struct position default_offset = { character, character, character, 
+    struct position default_offset = { character, character, character,
 				       0., 0., 0. };
     if (label) {
 	free(label->text);
@@ -1624,12 +1671,20 @@ unset_axislabel(AXIS_INDEX axis)
 void
 reset_command()
 {
+    int i;
     AXIS_INDEX axis;
     TBOOLEAN save_interactive = interactive;
-    static TBOOLEAN set_for_axis[AXIS_ARRAY_SIZE] = AXIS_ARRAY_INITIALIZER(TRUE);
 
     c_token++;
 
+    /* Reset session state as well as internal graphics state */
+    if (equals(c_token, "session")) {
+	clear_udf_list();
+	init_constants();
+	init_session();
+	return;
+    }
+
     /* Reset error state (only?) */
     update_gpval_variables(4);
     if (almost_equals(c_token,"err$orstate")) {
@@ -1646,6 +1701,12 @@ reset_command()
     }
 #endif
 
+    if (!(END_OF_COMMAND)) {
+	int_warn(c_token, "invalid option, expecting 'bind' or 'errorstate'");
+	while (!(END_OF_COMMAND))
+	    c_token++;
+    }
+
     /* Kludge alert, HBB 20000506: set to noninteractive mode, to
      * suppress some of the commentary output by the individual
      * unset_...() routines. */
@@ -1682,37 +1743,51 @@ reset_command()
     unset_axislabel_or_title(&title);
 
     reset_key();
-    unset_keytitle();
 
-    unset_timefmt();
-    unset_view(); /* has to be called in advance to reset reversed yrange if splot_map_active */
+    unset_view();
 
     for (axis=0; axis<AXIS_ARRAY_SIZE; axis++) {
-	SET_DEFFORMAT(axis, set_for_axis);
-	unset_timedata(axis);
+
+	AXIS default_axis_state = DEFAULT_AXIS_STRUCT;
+
+	/* Free contents before overwriting with default values */
+	free(axis_array[axis].formatstring);
+	if (axis_array[axis].link_udf) {
+	    free(axis_array[axis].link_udf->at);
+	    free(axis_array[axis].link_udf->definition);
+	    free(axis_array[axis].link_udf);
+	}
+	free_marklist(axis_array[axis].ticdef.def.user);
+	free(axis_array[axis].ticdef.font);
 	unset_zeroaxis(axis);
-	unset_range(axis);
+	unset_axislabel_or_title(&axis_array[axis].label);
+
+	memcpy(axis_array+axis, &default_axis_state, sizeof(AXIS));
+
 	unset_axislabel(axis);
+	axis_array[axis].formatstring = gp_strdup(DEF_FORMAT);
+	unset_timedata(axis);
+	unset_range(axis);
 
-	axis_array[axis].set_autoscale = AUTOSCALE_BOTH;
-	axis_array[axis].writeback_min = axis_array[axis].set_min
-	    = axis_defaults[axis].min;
-	axis_array[axis].writeback_max = axis_array[axis].set_max
-	    = axis_defaults[axis].max;
-
-	axis_array[axis].min_constraint = CONSTRAINT_NONE;
-	axis_array[axis].max_constraint = CONSTRAINT_NONE;
-	
 	/* 'tics' default is on for some, off for the other axes: */
 	unset_tics(axis);
-	axis_array[axis].ticmode = axis_defaults[axis].ticmode;
+	axis_array[axis].ticmode = axis_defaults[GPMIN(axis,PARALLEL_AXES)].ticmode;
 	unset_minitics(axis);
 	axis_array[axis].ticdef = default_axis_ticdef;
 	axis_array[axis].minitics = MINI_DEFAULT;
+	if (axis >= PARALLEL_AXES) {
+	    axis_array[axis].ticdef.rangelimited = TRUE;
+	    axis_array[axis].set_autoscale |= AUTOSCALE_FIXMIN | AUTOSCALE_FIXMAX;
+	}
+
+	axis_array[axis].linked_to_primary = FALSE;
 
 	reset_logscale(axis);
     }
     raxis = TRUE;
+    for (i=2; i<MAX_TICLEVEL; i++)
+	ticscale[i] = 1;
+    unset_timefmt();
 
     unset_boxplot();
     unset_boxwidth();
@@ -1725,6 +1800,7 @@ reset_command()
     draw_border = 31;
 
     draw_surface = TRUE;
+    implicit_surface = TRUE;
 
     data_style = POINTSTYLE;
     func_style = LINES;
@@ -1740,19 +1816,13 @@ reset_command()
     grid_lp = default_grid_lp;
     mgrid_lp = default_grid_lp;
     polar_grid_angle = 0;
-    grid_layer = -1;
+    grid_layer = LAYER_BEHIND;
 
-#ifdef VOLATILE_REFRESH
-    refresh_ok = 0;
-#endif
-    prefer_line_styles = FALSE;
+    SET_REFRESH_OK(E_REFRESH_NOT_OK, 0);
 
     reset_hidden3doptions();
     hidden3d = FALSE;
 
-    label_contours = TRUE;
-    strcpy(contour_format, "%8.3g");
-
     unset_angles();
     unset_mapping();
 
@@ -1764,6 +1834,7 @@ reset_command()
     unset_offsets();
     unset_contour();
     unset_cntrparam();
+    unset_cntrlabel();
     unset_zero();
     unset_dgrid3d();
     unset_ticslevel();
@@ -1779,13 +1850,30 @@ reset_command()
     df_unset_datafile_binary();
     unset_fillstyle();
     unset_histogram();
+#ifdef EAM_BOXED_TEXT
+    unset_textbox_style();
+#endif
+#ifdef BACKWARDS_COMPATIBLE
+    prefer_line_styles = FALSE;
+#endif
+
+#ifdef USE_MOUSE
+    mouse_setting = default_mouse_setting;
+#endif
 
     unset_missing();
-    df_separator = '\0';
+    free(df_separators);
+    df_separators = NULL;
     free(df_commentschars);
     df_commentschars = gp_strdup(DEFAULT_COMMENTS_CHARS);
 
-    unset_fit();
+    { /* Preserve some settings for `reset`, but not for `unset fit` */
+	verbosity_level save_verbosity = fit_verbosity;
+	TBOOLEAN save_errorscaling = fit_errorscaling;
+	unset_fit();
+	fit_verbosity = save_verbosity;
+	fit_errorscaling = save_errorscaling;
+    }
 
     update_gpval_variables(0); /* update GPVAL_ inner variables */
 
