@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util3d.c,v 1.44.2.1 2012/01/11 23:42:23 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util3d.c,v 1.48.2.1 2015/06/03 17:32:55 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util3d.c */
@@ -52,8 +52,9 @@ static char *RCSid() { return RCSid("$Id: util3d.c,v 1.44.2.1 2012/01/11 23:42:2
 #include "pm3d.h"
 #include "term_api.h"
 
-/* HBB 990826: all that stuff referenced from other modules is now
- * exported in graph3d.h, instead of being listed here */
+/* EAM DEBUG - moved these here from axis.h */
+#define AXIS_ACTUAL_MIN(axis) GPMIN(axis_array[axis].max, axis_array[axis].min)
+#define AXIS_ACTUAL_MAX(axis) GPMAX(axis_array[axis].max, axis_array[axis].min)
 
 /* Prototypes for local functions */
 static void mat_unit __PROTO((transform_matrix mat));
@@ -72,17 +73,6 @@ mat_unit(transform_matrix mat)
 		mat[i][j] = 0.0;
 }
 
-#if 0 /* HBB 990829: unused --> commented out */
-void
-mat_trans(double tx, double ty, double tz, transform_matrix mat)
-{
-    mat_unit(mat);		/* Make it unit matrix. */
-    mat[3][0] = tx;
-    mat[3][1] = ty;
-    mat[3][2] = tz;
-}
-#endif /* commented out */
-
 void
 mat_scale(double sx, double sy, double sz, transform_matrix mat)
 {
@@ -108,24 +98,6 @@ mat_rot_x(double teta, transform_matrix mat)
     mat[2][2] = cos_teta;
 }
 
-#if 0 /* HBB 990829: unused --> commented out */
-void
-mat_rot_y(double teta, transform_matrix mat)
-{
-    double cos_teta, sin_teta;
-
-    teta *= DEG2RAD;
-    cos_teta = cos(teta);
-    sin_teta = sin(teta);
-
-    mat_unit(mat);		/* Make it unit matrix. */
-    mat[0][0] = cos_teta;
-    mat[0][2] = -sin_teta;
-    mat[2][0] = sin_teta;
-    mat[2][2] = cos_teta;
-}
-#endif /* commented out */
-
 void
 mat_rot_z(double teta, transform_matrix mat)
 {
@@ -966,13 +938,17 @@ draw3d_line_unconditional(
     TERMCOORD(v2, x2, y2);
 
     /* Replace original color with the one passed in */
-	ls.pm3d_color = color;
+    ls.pm3d_color = color;
 
     /* Color by Z value */
     if (ls.pm3d_color.type == TC_Z)
 	    ls.pm3d_color.value = (v1->real_z + v2->real_z) * 0.5;
 
-    term_apply_lp_properties(&ls);
+    /* This interrupts the polyline, messing up dash patterns. */
+    /* Skip it if the caller indicates that the line properties are */
+    /* already set by passing in color.type = TC_DEFAULT */
+    if (color.type != TC_DEFAULT)
+	term_apply_lp_properties(&ls);
 
     /* Support for hidden3d VECTOR mode with arrowheads */
     if (lp->p_type == PT_ARROWHEAD)
@@ -1036,32 +1012,16 @@ polyline3d_start(p_vertex v1)
 void
 polyline3d_next(p_vertex v2, struct lp_style_type *lp)
 {
-    unsigned int x1, y1;
-    unsigned int x2, y2;
+    /* v5: Indicate that line properties are already set so that draw3d_* */
+    /*     routines do not mess up dash patterns by resetting them        */
+    t_colorspec nochange = DEFAULT_COLORSPEC;
 
-    /* Copied from draw3d_line(): */
-    /* FIXME HBB 20031218: hidden3d mode will still create isolated
-     * edges! */
-    if (hidden3d && draw_surface) {
+    if (hidden3d && draw_surface)
 	draw_line_hidden(&polyline3d_previous_vertex, v2, lp);
-	polyline3d_previous_vertex = *v2;
-	return;
-    }
-
-    /* Copied from draw3d_line_unconditional: */
-    /* If use_palette is active, polylines can't be used -->
-     * revert back to old method */
-    if (lp->use_palette) {
+    else
 	draw3d_line_unconditional(&polyline3d_previous_vertex, v2,
-				  lp, lp->pm3d_color);
-	polyline3d_previous_vertex = *v2;
-	return;
+				  lp, nochange);
 
-    }
-
-    TERMCOORD(&polyline3d_previous_vertex, x1, y1);
-    TERMCOORD(v2, x2, y2);
-    draw_clip_line(x1,y1,x2,y2);
     polyline3d_previous_vertex = *v2;
 }
 
